// 
// Stefan Weinzierl
//

#include <iostream>
#include <stdexcept>
#include <ctime>

#include "partonevolution/partonevolution.h"

using namespace std;
using namespace pdf;

void check(const partondistribution & u, 
	   const partondistribution & d, 
	   const partondistribution & sigmadistr, 
	   const partondistribution & gluondistr, 
	   const partondistribution & Delta_2, 
	   const partondistribution & Delta_3, 
	   const partondistribution & Delta_4, 
	   const partondistribution & u_qed, 
	   const partondistribution & d_qed, 
	   const partondistribution & Delta_UD, 
	   const partondistribution & Delta_ds, 
	   const partondistribution & Delta_uc, 
	   const partondistribution & photondistr, 
	   const partondistribution & leptondistr, 
	   const partondistribution & Delta_UD_noqed, 
	   const partondistribution & Delta_ds_noqed, 
	   const partondistribution & Delta_uc_noqed, 
	   double Q, double x, int n_points);

int main()
{

  try{

    std::cout << std::endl;
    std::cout << "Evolution of parton distributions with QED corrections" << std::endl;
    std::cout << "(Uses the x-space method and iterates the kernel, therefore the method is rather slow.)" << std::endl;
    std::cout << std::endl;

    alpha_strong::method = alpha_strong::truncate_in_one_over_L;
    evolutionkernel::method = evolutionkernel::x_space_truncate_in_one_over_L;
    evolutionkernel::number_of_iterations = 100000;

    alpha_strong::charm_threshold  = 0.0;
    alpha_strong::bottom_threshold = 1E10;

    alpha_strong::lambda_3 = 0.250;
    alpha_strong::lambda_4 = 0.250;
    alpha_strong::lambda_5 = 0.250;

    alpha_finestruc::charm_threshold  = 0.0;
    alpha_finestruc::tau_threshold    = 1.777;
    alpha_finestruc::bottom_threshold = 1E10;

    // corresponds to alpha = 1/133.40
    alpha_finestruc::value_at_tau = 0.0005965327699;

    double iniscale = 2.0;

    // ------------------------------------------------------------------------

    // u- valence
    double A_u[1]     = { 35/16.};
    double alpha_u[1] = { 0.5 };
    double beta_u[1]  = { 3.0 };

    partondistribution uvalence =
      partondistribution(1,iniscale,A_u,alpha_u,beta_u,-1);

    partondistribution_with_qed uvalence_with_qed =
      partondistribution_with_qed(1,iniscale,A_u,alpha_u,beta_u,2.0/3.0,-1);

    // -----------------------------------------------------------------
    // d-valence
    double A_d[1]     = { 315/256.};
    double alpha_d[1] = { 0.5 };
    double beta_d[1]  = { 4.0 };

    partondistribution dvalence =
      partondistribution(1,iniscale,A_d,alpha_d,beta_d,-1);

    partondistribution_with_qed dvalence_with_qed =
      partondistribution_with_qed(1,iniscale,A_d,alpha_d,beta_d,-1.0/3.0,-1);


    // -----------------------------------------------------------------
    // gluon distribution
    double A_g[1]     = { 1.90836};
    double alpha_g[1] = { -0.2 };
    double beta_g[1]  = { 5.0 };

    partondistribution gluondistr =
      partondistribution(1,iniscale,A_g,alpha_g,beta_g);

    // ------------------------------------------------------------------
    // quark-singlet combination
    double A_sing[3]     = { 0.673345, 35/16., 315/256.};
    double alpha_sing[3] = { -0.2,     0.5,    0.5 };
    double beta_sing[3]  = {  7.0,     3.0,    4.0 };

    partondistribution sigmadistr =
      partondistribution(3,iniscale,A_sing,alpha_sing,beta_sing);

    // ------------------------------------------------------------------
    // non-singlet u+ubar-d-dbar
    // non-singlet u+ubar+d+dbar - 2 ( s + sbar )
    // non-singlet u+ubar+d+dabar+s+sbar - 3 ( c + cbar)
    double A_D2[2]     = { 35/16., -315/256. };
    double alpha_D2[2] = { 0.5 , 0.5 };
    double beta_D2[2]  = { 3.0 , 4.0 };

    double A_D3[2]     = { 35/16., 315/256. };
    double alpha_D3[2] = { 0.5 , 0.5 };
    double beta_D3[2]  = { 3.0 , 4.0 };

    partondistribution Delta_2 =
      partondistribution(2,iniscale,A_D2,alpha_D2,beta_D2);

    partondistribution Delta_3 =
      partondistribution(2,iniscale,A_D3,alpha_D3,beta_D3);

    partondistribution Delta_4 =
      partondistribution(3,iniscale,A_sing,alpha_sing,beta_sing);

    // ------------------------------------------------------------------
    double A_empty[0];
    double alpha_empty[0];
    double beta_empty[0];

    partondistribution photondistr =
      partondistribution(0,iniscale,A_empty,alpha_empty,beta_empty);

    partondistribution leptondistr =
      partondistribution(0,iniscale,A_empty,alpha_empty,beta_empty);

    // ------------------------------------------------------------------
    double A_UD[3]     = { (-1./3.)*0.673345, 35/16., -315/256.};

    partondistribution_with_qed Delta_UD =
      partondistribution_with_qed(3,iniscale,A_UD,alpha_sing,beta_sing);

    partondistribution Delta_UD_noqed =
      partondistribution(3,iniscale,A_UD,alpha_sing,beta_sing);

    // ------------------------------------------------------------------
    double A_uc[2]     = { (1./3.)*0.673345, 35/16.};
    double alpha_uc[2] = { -0.2,     0.5 };
    double beta_uc[2]  = {  7.0,     3.0 };

    partondistribution_with_qed Delta_uc =
      partondistribution_with_qed(2,iniscale,A_uc,alpha_sing,beta_sing,2.0/3.0);

    partondistribution Delta_uc_noqed =
      partondistribution(2,iniscale,A_uc,alpha_sing,beta_sing);

    // ------------------------------------------------------------------

    partondistribution_with_qed Delta_ds =
      partondistribution_with_qed(1,iniscale,A_d,alpha_d,beta_d,-1.0/3.0);

    partondistribution Delta_ds_noqed =
      partondistribution(1,iniscale,A_d,alpha_d,beta_d);

    // ------------------------------------------------------------------
    // evolution

    double Q = 10.0;
    double x;

    x = 0.00001;
    check(uvalence,dvalence,sigmadistr,gluondistr,Delta_2,Delta_3,Delta_4,
	  uvalence_with_qed,dvalence_with_qed,Delta_UD,Delta_ds,Delta_uc,photondistr,leptondistr,
	  Delta_UD_noqed,Delta_ds_noqed,Delta_uc_noqed,Q,x,30);

    x = 0.0001;
    check(uvalence,dvalence,sigmadistr,gluondistr,Delta_2,Delta_3,Delta_4,
	  uvalence_with_qed,dvalence_with_qed,Delta_UD,Delta_ds,Delta_uc,photondistr,leptondistr,
	  Delta_UD_noqed,Delta_ds_noqed,Delta_uc_noqed,Q,x,30);

    x = 0.001;
    check(uvalence,dvalence,sigmadistr,gluondistr,Delta_2,Delta_3,Delta_4,
	  uvalence_with_qed,dvalence_with_qed,Delta_UD,Delta_ds,Delta_uc,photondistr,leptondistr,
	  Delta_UD_noqed,Delta_ds_noqed,Delta_uc_noqed,Q,x,30);

    x = 0.01;
    check(uvalence,dvalence,sigmadistr,gluondistr,Delta_2,Delta_3,Delta_4,
	  uvalence_with_qed,dvalence_with_qed,Delta_UD,Delta_ds,Delta_uc,photondistr,leptondistr,
	  Delta_UD_noqed,Delta_ds_noqed,Delta_uc_noqed,Q,x,30);

    x = 0.1;
    check(uvalence,dvalence,sigmadistr,gluondistr,Delta_2,Delta_3,Delta_4,
	  uvalence_with_qed,dvalence_with_qed,Delta_UD,Delta_ds,Delta_uc,photondistr,leptondistr,
	  Delta_UD_noqed,Delta_ds_noqed,Delta_uc_noqed,Q,x,30);

    x = 0.3;
    check(uvalence,dvalence,sigmadistr,gluondistr,Delta_2,Delta_3,Delta_4,
	  uvalence_with_qed,dvalence_with_qed,Delta_UD,Delta_ds,Delta_uc,photondistr,leptondistr,
	  Delta_UD_noqed,Delta_ds_noqed,Delta_uc_noqed,Q,x,30);

    x = 0.7;
    check(uvalence,dvalence,sigmadistr,gluondistr,Delta_2,Delta_3,Delta_4,
	  uvalence_with_qed,dvalence_with_qed,Delta_UD,Delta_ds,Delta_uc,photondistr,leptondistr,
	  Delta_UD_noqed,Delta_ds_noqed,Delta_uc_noqed,Q,x,30);

    std::cout << std::endl;


  } catch (std::exception &e)
    {
      std::cout << "Exception : " << e.what() << std::endl;
    }

  return 0;
}

void check(const partondistribution & u, 
	   const partondistribution & d, 
	   const partondistribution & s, 
	   const partondistribution & g, 
	   const partondistribution & D_2, 
	   const partondistribution & D_3, 
	   const partondistribution & D_4, 
	   const partondistribution & u_qed, 
	   const partondistribution & d_qed, 
	   const partondistribution & D_UD, 
	   const partondistribution & D_ds, 
	   const partondistribution & D_uc, 
	   const partondistribution & phot, 
	   const partondistribution & lept, 
	   const partondistribution & D_UD_noqed, 
	   const partondistribution & D_ds_noqed, 
	   const partondistribution & D_uc_noqed, 
	   double Q, double x, int n_points)
    {
      double u_v,d_v,u_v_qed,d_v_qed,Delta_4,Delta_2,Delta_3;

      double Delta_UD, Sigma_qed, gluon_qed, photon, lepton, Delta_ds, Delta_ds_noqed, Delta_uc, Delta_uc_noqed;

      double Sigma,gluon,Sea,c;

      double Sea_qed, c_qed, Delta_4_qed;

      double Delta_UD_noqed, Sigma_noqed, gluon_noqed, photon_noqed, lepton_noqed, Sea_noqed, Delta_4_noqed, c_noqed;

      double u_withqed, u_noqed, d_withqed, d_noqed;
      double ubar_withqed, ubar_noqed, dbar_withqed, dbar_noqed;

      vector_d singlet_qcd(2);
      vector_d singlet_qcd_with_qed(5);
      vector_d singlet_qcd_noqed(5);

      // evolve
      u_v = evolve_nonsinglet(u,Q,x,n_points);
      d_v = evolve_nonsinglet(d,Q,x,n_points);
      u_v_qed = evolve_nonsinglet(u_qed,Q,x,n_points);
      d_v_qed = evolve_nonsinglet(d_qed,Q,x,n_points);

      Delta_2 = evolve_nonsinglet(D_2,Q,x,n_points);
      Delta_3 = evolve_nonsinglet(D_3,Q,x,n_points);
      Delta_4 = evolve_nonsinglet(D_4,Q,x,n_points);

      Delta_ds = evolve_nonsinglet(D_ds,Q,x,n_points);
      Delta_ds_noqed = evolve_nonsinglet(D_ds_noqed,Q,x,n_points);

      Delta_uc = evolve_nonsinglet(D_uc,Q,x,n_points);
      Delta_uc_noqed = evolve_nonsinglet(D_uc_noqed,Q,x,n_points);

      singlet_qcd = evolve_singlet(s,g,Q,x,n_points);
      Sigma = singlet_qcd[0];
      gluon = singlet_qcd[1];

      singlet_qcd_with_qed = evolve_singlet_with_qed(D_UD,s,g,phot,lept,Q,x,n_points);
      Delta_UD  = singlet_qcd_with_qed[0];
      Sigma_qed = singlet_qcd_with_qed[1];
      gluon_qed = singlet_qcd_with_qed[2];
      photon    = singlet_qcd_with_qed[3];
      lepton    = singlet_qcd_with_qed[4];

      singlet_qcd_noqed = evolve_singlet_with_qed(D_UD_noqed,s,g,phot,lept,Q,x,n_points);
      Delta_UD_noqed  = singlet_qcd_noqed[0];
      Sigma_noqed     = singlet_qcd_noqed[1];
      gluon_noqed     = singlet_qcd_noqed[2];
      photon_noqed    = singlet_qcd_noqed[3];
      lepton_noqed    = singlet_qcd_noqed[4];

      // derived quantities
      Sea = Sigma - u_v - d_v;
      Sea_qed = Sigma_qed - u_v_qed - d_v_qed;
      Sea_noqed = Sigma_noqed - u_v - d_v;

      c = ( Sigma - Delta_4 ) / 8.0;

      Delta_4_qed = -Delta_UD + 2.0*Delta_uc;
      c_qed = ( Sigma_qed - Delta_4_qed ) / 8.0;

      Delta_4_noqed = -Delta_UD_noqed + 2.0*Delta_uc_noqed;
      c_noqed = ( Sigma_noqed - Delta_4_noqed ) / 8.0;

      //
      u_withqed = 0.5*u_v_qed + 1/8.0*Delta_UD + 0.25*Delta_uc + 1/8.0*Sigma_qed;
      u_noqed = 0.5*u_v + 1/8.0*Delta_UD_noqed + 0.25*Delta_uc_noqed + 1/8.0*Sigma_noqed;

      ubar_withqed = -0.5*u_v_qed + 1/8.0*Delta_UD + 0.25*Delta_uc + 1/8.0*Sigma_qed;
      ubar_noqed = -0.5*u_v + 1/8.0*Delta_UD_noqed + 0.25*Delta_uc_noqed + 1/8.0*Sigma_noqed;

      d_withqed = 0.5*d_v_qed - 1/8.0*Delta_UD + 0.25*Delta_ds + 1/8.0*Sigma_qed;
      d_noqed = 0.5*d_v - 1/8.0*Delta_UD_noqed + 0.25*Delta_ds_noqed + 1/8.0*Sigma_noqed;

      dbar_withqed = -0.5*d_v_qed - 1/8.0*Delta_UD + 0.25*Delta_ds + 1/8.0*Sigma_qed;
      dbar_noqed = -0.5*d_v - 1/8.0*Delta_UD_noqed + 0.25*Delta_ds_noqed + 1/8.0*Sigma_noqed;

      //
      std::cout << " n = " << n_points 
	   << ", Q = " << Q 
	   << ", x = " << x
	   << std::endl; 
      std::cout << " x u_v = " << x*u_v_qed 
	   << ", x d_v = " << x*d_v_qed 
	   << ", x Sigma = " << x*Sigma_qed 
	   << ", x S = " << x*Sea_qed 
	   << ", 2 x c = " << 2*x*c_qed 
	   << ", x g = " << x*gluon_qed 
	   << std::endl;
      std::cout << " x photon = " << x*photon 
	   << ", x lepton = " << x*lepton 
	   << ", x Delta_UD = " << x*Delta_UD 
	   << std::endl;
      std::cout << std::endl;

    }

