

/** @file Gauss_Laguerre.cc
 *
 * Implementation of Gauss_Laguerre 
 *
 */

/*
 *  Copyright (C) 2002 Stefan Weinzierl
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <cmath>
#include <stdexcept>

#include "matrix_d.h"

#include "Gauss_Laguerre.h"
#include "partondistribution.h"
#include "integration_contour.h"

namespace pdf {

  /**
   *
   * Entry point for the evolution of a non-singlet parton distribution.
   *
   * The actual computation is passed to evolve_nonsinglet_n, where n is the number
   * of points where the Gauss Laguerre quadrature formula is evaluated.
   * Formulae for n = 3, 5, 10, 20 and 30 are implemented. The default choice
   * uses 5 points. 
   *
   */
  double evolve_nonsinglet(const partondistribution & f, double Q, double x, int number_of_points)
    {
      if ( number_of_points == 3 ) return( evolve_nonsinglet_3(f,Q,x) );
      else if ( number_of_points == 5 ) return( evolve_nonsinglet_5(f,Q,x) );
      else if ( number_of_points == 10 ) return( evolve_nonsinglet_10(f,Q,x) );
      else if ( number_of_points == 20 ) return( evolve_nonsinglet_20(f,Q,x) );
      else if ( number_of_points == 30 ) return( evolve_nonsinglet_30(f,Q,x) );

      throw(std::logic_error("no Gauss-Laguerre quadrature formula for the requested number of points implemented"));
    }

  /**
   *
   * Evaluation of the inverse Mellin transform for a non-singlet parton
   * distribution with a Gauss-Laguerre quadrature formula with 3 points.
   *
   */
  double evolve_nonsinglet_3(const partondistribution & f, double Q, double x)
    {
      return ( 1.449259190449*integration_contour::nonsinglet(f,Q,x,0.190163509193)
	       +0.314134640646*integration_contour::nonsinglet(f,Q,x,1.784492748543)
	       +0.009060019811*integration_contour::nonsinglet(f,Q,x,5.525343742263)
	       );
    }

  /**
   *
   * Evaluation of the inverse Mellin transform for a non-singlet parton
   * distribution with a Gauss-Laguerre quadrature formula with 5 points.
   *
   */
  double evolve_nonsinglet_5(const partondistribution & f, double Q, double x)
    {
      return ( 1.221725267*integration_contour::nonsinglet(f,Q,x,0.117581320)
	       +0.480277222*integration_contour::nonsinglet(f,Q,x,1.074562012)
	       +0.067748788*integration_contour::nonsinglet(f,Q,x,3.085937444)
	       +0.00268729149*integration_contour::nonsinglet(f,Q,x,6.414729734)
	       +0.0000152808657*integration_contour::nonsinglet(f,Q,x,11.807189490) 
	       );
    }

  /**
   *
   * Evaluation of the inverse Mellin transform for a non-singlet parton
   * distribution with a Gauss-Laguerre quadrature formula with 10 points.
   *
   */
  double evolve_nonsinglet_10(const partondistribution & f, double Q, double x)
    {
      return ( 0.9244873392012*integration_contour::nonsinglet(f,Q,x,0.0601920631495879)
	       +0.5733510107257*integration_contour::nonsinglet(f,Q,x,0.5438675002946460)
	       +0.2180344120400*integration_contour::nonsinglet(f,Q,x,1.5229441054044437)
	       +0.04962104177493*integration_contour::nonsinglet(f,Q,x,3.02251337645157398)
	       +0.006487546684475*integration_contour::nonsinglet(f,Q,x,5.08490775009852397)
	       +0.0004566772720327*integration_contour::nonsinglet(f,Q,x,7.7774392315254451)
	       +0.00001560511295706*integration_contour::nonsinglet(f,Q,x,11.208130204348663)
	       +0.21721387415386E-6*integration_contour::nonsinglet(f,Q,x,15.561163332189350)
	       +0.87986819845464E-9*integration_contour::nonsinglet(f,Q,x,21.193892096301541)
	       +0.44587872910683E-12*integration_contour::nonsinglet(f,Q,x,29.024950340236226) 
	       );
    }

  /**
   *
   * Evaluation of the inverse Mellin transform for a non-singlet parton
   * distribution with a Gauss-Laguerre quadrature formula with 20 points.
   *
   */
  double evolve_nonsinglet_20(const partondistribution & f, double Q, double x)
    {
      double s1 = 0.6772865549*integration_contour::nonsinglet(f,Q,x,0.3046323928E-1)
	+0.5314565038*integration_contour::nonsinglet(f,Q,x,0.2744447158)+0.3267574654*integration_contour::nonsinglet(f,Q,x,0.7638875584)
	+0.1569492117*integration_contour::nonsinglet(f,Q,x,0.1501801498E1)+0.5862513107E-1*integration_contour::nonsinglet(f,Q,x,0.2492830145E1)
	+0.1692177602E-1*integration_contour::nonsinglet(f,Q,x,0.3743418041E1)+0.3742993659E-2*integration_contour::nonsinglet(f,Q,x,0.5262055854E1)
	+0.6277071891E-3*integration_contour::nonsinglet(f,Q,x,0.7059627736E1)+0.7873867962E-4*integration_contour::nonsinglet(f,Q,x,0.9149898312E1)
	+0.7263152301E-5*integration_contour::nonsinglet(f,Q,x,0.1155019829E2);
      double t0 = s1+0.4822288327E-6*integration_contour::nonsinglet(f,Q,x,0.1428240369E2)
	+0.2242472166E-7*integration_contour::nonsinglet(f,Q,x,0.1737436698E2)+0.7051241583E-9*integration_contour::nonsinglet(f,Q,x,0.2086207519E2)
	+0.1431305611E-10*integration_contour::nonsinglet(f,Q,x,0.2479303989E2)+0.1761141529E-12*integration_contour::nonsinglet(f,Q,x,0.2923191016E2)
	+0.1201671758E-14*integration_contour::nonsinglet(f,Q,x,0.3427042893E2)+0.3978362024E-17*integration_contour::nonsinglet(f,Q,x,0.4004681579E2)
	+0.5135186731E-20*integration_contour::nonsinglet(f,Q,x,0.4678884639E2)+0.1708811393E-23*integration_contour::nonsinglet(f,Q,x,0.5493155562E2)
	+0.5182087428E-28*integration_contour::nonsinglet(f,Q,x,0.6558993199E2);
      return (t0);
    }

  /**
   *
   * Evaluation of the inverse Mellin transform for a non-singlet parton
   * distribution with a Gauss-Laguerre quadrature formula with 30 points.
   *
   */
  double evolve_nonsinglet_30(const partondistribution & f, double Q, double x)
    {
      double s1 = 0.4757378099*integration_contour::nonsinglet(f,Q,x,0.183612305)
	+0.559706235*integration_contour::nonsinglet(f,Q,x,0.2039219378E-1)+0.3435523138*integration_contour::nonsinglet(f,Q,x,0.5104942191)
	+0.2105975274*integration_contour::nonsinglet(f,Q,x,0.1001927927E1)+0.4812254553E-1*integration_contour::nonsinglet(f,Q,x,0.2484340278E1)
	+0.1094379419*integration_contour::nonsinglet(f,Q,x,0.1659265178E1)+0.1786435672E-1*integration_contour::nonsinglet(f,Q,x,0.3479499028E1)
	+0.5582649658E-2*integration_contour::nonsinglet(f,Q,x,0.4647636927E1)+0.3205546694E-3*integration_contour::nonsinglet(f,Q,x,0.7517487793E1)
	+0.1463547114E-2*integration_contour::nonsinglet(f,Q,x,0.5992248202E1)+0.5837483808E-4*integration_contour::nonsinglet(f,Q,x,0.9228249122E1)
	+0.1086703227E-5*integration_contour::nonsinglet(f,Q,x,0.1323021228E2)+0.8788573873E-5*integration_contour::nonsinglet(f,Q,x,0.1113026149E2)
	+0.1095110924E-6*integration_contour::nonsinglet(f,Q,x,0.1553590102E2)+0.891364555E-8*integration_contour::nonsinglet(f,Q,x,0.1805643521E2);
      double t0 = s1+0.5798718026E-9*integration_contour::nonsinglet(f,Q,x,0.2080248162E2)
	+0.2977914698E-10*integration_contour::nonsinglet(f,Q,x,0.2378659288E2)+0.1189686103E-11*integration_contour::nonsinglet(f,Q,x,0.2702363844E2)
	+0.3633249153E-13*integration_contour::nonsinglet(f,Q,x,0.3053138327E2)+0.8304888219E-15*integration_contour::nonsinglet(f,Q,x,0.3433128161E2)
	+0.1384649582E-16*integration_contour::nonsinglet(f,Q,x,0.3844959272E2)+0.1630728095E-18*integration_contour::nonsinglet(f,Q,x,0.4291899667E2)
	+0.1302513451E-20*integration_contour::nonsinglet(f,Q,x,0.4778101845E2)+0.6691513912E-23*integration_contour::nonsinglet(f,Q,x,0.5308982661E2)
	+0.2058811994E-25*integration_contour::nonsinglet(f,Q,x,0.5891851875E2)+0.3431146295E-28*integration_contour::nonsinglet(f,Q,x,0.6537027557E2)
	+0.2665119224E-31*integration_contour::nonsinglet(f,Q,x,0.7260010093E2)+0.4879495176E-39*integration_contour::nonsinglet(f,Q,x,0.9064760683E2)
	+0.7543253454E-35*integration_contour::nonsinglet(f,Q,x,0.8086322182E2)+0.2219174496E-44*integration_contour::nonsinglet(f,Q,x,0.1032075007E3);
      return(t0);
    }

  /**
   *
   * Entry point for the evolution of singlet parton distribution.
   * g is a parameterization of the gluon distribution, whereas 
   * s is a parameterization for
   * \f[
   *    \Sigma = \sum\limits_{q=u,d,...} \left( q + \bar{q} \right)
   * \f]
   *
   * The actual computation is passed to evolve_nonsinglet_n, where n is the number
   * of points where the Gauss Laguerre quadrature formula is evaluated.
   * Formulae for n = 3, 5, 10, 20 and 30 are implemented. The default choice
   * uses 5 points. 
   *
   */
  vector_d evolve_singlet(const partondistribution & s, const partondistribution & g, double Q, double x, int number_of_points)
    {
      if ( number_of_points == 3 ) return( evolve_singlet_3(s,g,Q,x) );
      else if ( number_of_points == 5 ) return( evolve_singlet_5(s,g,Q,x) );
      else if ( number_of_points == 10 ) return( evolve_singlet_10(s,g,Q,x) );
      else if ( number_of_points == 20 ) return( evolve_singlet_20(s,g,Q,x) );
      else if ( number_of_points == 30 ) return( evolve_singlet_30(s,g,Q,x) );

      throw(std::logic_error("no Gauss-Laguerre quadrature formula for the requested number of points implemented"));
    }

  /**
   *
   * Evaluation of the inverse Mellin transform for singlet parton
   * distributions with a Gauss-Laguerre quadrature formula with 3 points.
   *
   */
  vector_d evolve_singlet_3(const partondistribution & s, const partondistribution & g, double Q, double x)
    {
      return ( 1.449259190449*integration_contour::singlet(s,g,Q,x,0.190163509193)
	       +0.314134640646*integration_contour::singlet(s,g,Q,x,1.784492748543)
	       +0.009060019811*integration_contour::singlet(s,g,Q,x,5.525343742263)
	       );
    }

  /**
   *
   * Evaluation of the inverse Mellin transform for singlet parton
   * distributions with a Gauss-Laguerre quadrature formula with 5 points.
   *
   */
  vector_d evolve_singlet_5(const partondistribution & s, const partondistribution & g, double Q, double x)
    {
      return ( 1.221725267*integration_contour::singlet(s,g,Q,x,0.117581320)
	       +0.480277222*integration_contour::singlet(s,g,Q,x,1.074562012)
	       +0.067748788*integration_contour::singlet(s,g,Q,x,3.085937444)
	       +0.00268729149*integration_contour::singlet(s,g,Q,x,6.414729734)
	       +0.0000152808657*integration_contour::singlet(s,g,Q,x,11.807189490) 
	       );
    }

  /**
   *
   * Evaluation of the inverse Mellin transform for singlet parton
   * distributions with a Gauss-Laguerre quadrature formula with 10 points.
   *
   */
  vector_d evolve_singlet_10(const partondistribution & s, const partondistribution & g, double Q, double x)
    {
      return ( 0.9244873392012*integration_contour::singlet(s,g,Q,x,0.0601920631495879)
	       +0.5733510107257*integration_contour::singlet(s,g,Q,x,0.5438675002946460)
	       +0.2180344120400*integration_contour::singlet(s,g,Q,x,1.5229441054044437)
	       +0.04962104177493*integration_contour::singlet(s,g,Q,x,3.02251337645157398)
	       +0.006487546684475*integration_contour::singlet(s,g,Q,x,5.08490775009852397)
	       +0.0004566772720327*integration_contour::singlet(s,g,Q,x,7.7774392315254451)
	       +0.00001560511295706*integration_contour::singlet(s,g,Q,x,11.208130204348663)
	       +0.21721387415386E-6*integration_contour::singlet(s,g,Q,x,15.561163332189350)
	       +0.87986819845464E-9*integration_contour::singlet(s,g,Q,x,21.193892096301541)
	       +0.44587872910683E-12*integration_contour::singlet(s,g,Q,x,29.024950340236226) 
	       );
    }

  /**
   *
   * Evaluation of the inverse Mellin transform for singlet parton
   * distributions with a Gauss-Laguerre quadrature formula with 20 points.
   *
   */
  vector_d evolve_singlet_20(const partondistribution & s, const partondistribution & g, double Q, double x)
    {
      vector_d s1 = 0.6772865549*integration_contour::singlet(s,g,Q,x,0.3046323928E-1)
	+0.5314565038*integration_contour::singlet(s,g,Q,x,0.2744447158)+0.3267574654*integration_contour::singlet(s,g,Q,x,0.7638875584)
	+0.1569492117*integration_contour::singlet(s,g,Q,x,0.1501801498E1)+0.5862513107E-1*integration_contour::singlet(s,g,Q,x,0.2492830145E1)
	+0.1692177602E-1*integration_contour::singlet(s,g,Q,x,0.3743418041E1)+0.3742993659E-2*integration_contour::singlet(s,g,Q,x,0.5262055854E1)
	+0.6277071891E-3*integration_contour::singlet(s,g,Q,x,0.7059627736E1)+0.7873867962E-4*integration_contour::singlet(s,g,Q,x,0.9149898312E1)
	+0.7263152301E-5*integration_contour::singlet(s,g,Q,x,0.1155019829E2);
      vector_d t0 = s1+0.4822288327E-6*integration_contour::singlet(s,g,Q,x,0.1428240369E2)
	+0.2242472166E-7*integration_contour::singlet(s,g,Q,x,0.1737436698E2)+0.7051241583E-9*integration_contour::singlet(s,g,Q,x,0.2086207519E2)
	+0.1431305611E-10*integration_contour::singlet(s,g,Q,x,0.2479303989E2)+0.1761141529E-12*integration_contour::singlet(s,g,Q,x,0.2923191016E2)
	+0.1201671758E-14*integration_contour::singlet(s,g,Q,x,0.3427042893E2)+0.3978362024E-17*integration_contour::singlet(s,g,Q,x,0.4004681579E2)
	+0.5135186731E-20*integration_contour::singlet(s,g,Q,x,0.4678884639E2)+0.1708811393E-23*integration_contour::singlet(s,g,Q,x,0.5493155562E2)
	+0.5182087428E-28*integration_contour::singlet(s,g,Q,x,0.6558993199E2);
      return (t0);
    }

  /**
   *
   * Evaluation of the inverse Mellin transform for singlet parton
   * distributions with a Gauss-Laguerre quadrature formula with 30 points.
   *
   */
  vector_d evolve_singlet_30(const partondistribution & s, const partondistribution & g, double Q, double x)
    {
      vector_d s1 = 0.4757378099*integration_contour::singlet(s,g,Q,x,0.183612305)
	+0.559706235*integration_contour::singlet(s,g,Q,x,0.2039219378E-1)+0.3435523138*integration_contour::singlet(s,g,Q,x,0.5104942191)
	+0.2105975274*integration_contour::singlet(s,g,Q,x,0.1001927927E1)+0.4812254553E-1*integration_contour::singlet(s,g,Q,x,0.2484340278E1)
	+0.1094379419*integration_contour::singlet(s,g,Q,x,0.1659265178E1)+0.1786435672E-1*integration_contour::singlet(s,g,Q,x,0.3479499028E1)
	+0.5582649658E-2*integration_contour::singlet(s,g,Q,x,0.4647636927E1)+0.3205546694E-3*integration_contour::singlet(s,g,Q,x,0.7517487793E1)
	+0.1463547114E-2*integration_contour::singlet(s,g,Q,x,0.5992248202E1)+0.5837483808E-4*integration_contour::singlet(s,g,Q,x,0.9228249122E1)
	+0.1086703227E-5*integration_contour::singlet(s,g,Q,x,0.1323021228E2)+0.8788573873E-5*integration_contour::singlet(s,g,Q,x,0.1113026149E2)
	+0.1095110924E-6*integration_contour::singlet(s,g,Q,x,0.1553590102E2)+0.891364555E-8*integration_contour::singlet(s,g,Q,x,0.1805643521E2);
      vector_d t0 = s1+0.5798718026E-9*integration_contour::singlet(s,g,Q,x,0.2080248162E2)
	+0.2977914698E-10*integration_contour::singlet(s,g,Q,x,0.2378659288E2)+0.1189686103E-11*integration_contour::singlet(s,g,Q,x,0.2702363844E2)
	+0.3633249153E-13*integration_contour::singlet(s,g,Q,x,0.3053138327E2)+0.8304888219E-15*integration_contour::singlet(s,g,Q,x,0.3433128161E2)
	+0.1384649582E-16*integration_contour::singlet(s,g,Q,x,0.3844959272E2)+0.1630728095E-18*integration_contour::singlet(s,g,Q,x,0.4291899667E2)
	+0.1302513451E-20*integration_contour::singlet(s,g,Q,x,0.4778101845E2)+0.6691513912E-23*integration_contour::singlet(s,g,Q,x,0.5308982661E2)
	+0.2058811994E-25*integration_contour::singlet(s,g,Q,x,0.5891851875E2)+0.3431146295E-28*integration_contour::singlet(s,g,Q,x,0.6537027557E2)
	+0.2665119224E-31*integration_contour::singlet(s,g,Q,x,0.7260010093E2)+0.4879495176E-39*integration_contour::singlet(s,g,Q,x,0.9064760683E2)
	+0.7543253454E-35*integration_contour::singlet(s,g,Q,x,0.8086322182E2)+0.2219174496E-44*integration_contour::singlet(s,g,Q,x,0.1032075007E3);
      return(t0);
    }


  /**
   *
   * Entry point for the evolution of the singlet parton distribution with QED effects.
   *
   * The actual computation is passed to evolve_nonsinglet_n, where n is the number
   * of points where the Gauss Laguerre quadrature formula is evaluated.
   * Formulae for n = 3, 5, 10, 20 and 30 are implemented. The default choice
   * uses 5 points. 
   *
   */
  vector_d evolve_singlet_with_qed(const partondistribution & Delta, 
				   const partondistribution & s, const partondistribution & g, 
				   const partondistribution & phot, const partondistribution & lept, 
				   double Q, double x, int number_of_points)
    {
      if ( number_of_points == 3 ) return( evolve_singlet_with_qed_3(Delta,s,g,phot,lept,Q,x) );
      else if ( number_of_points == 5 ) return( evolve_singlet_with_qed_5(Delta,s,g,phot,lept,Q,x) );
      else if ( number_of_points == 10 ) return( evolve_singlet_with_qed_10(Delta,s,g,phot,lept,Q,x) );
      else if ( number_of_points == 20 ) return( evolve_singlet_with_qed_20(Delta,s,g,phot,lept,Q,x) );
      else if ( number_of_points == 30 ) return( evolve_singlet_with_qed_30(Delta,s,g,phot,lept,Q,x) );

      throw(std::logic_error("no Gauss-Laguerre quadrature formula for the requested number of points implemented"));
    }

  /**
   *
   * Evaluation of the inverse Mellin transform for singlet parton
   * distributions with QED effects with a Gauss-Laguerre quadrature formula with 3 points.
   *
   */
  vector_d evolve_singlet_with_qed_3(const partondistribution & Delta, 
				     const partondistribution & s, const partondistribution & g, 
				     const partondistribution & phot, const partondistribution & lept, 
				     double Q, double x)
    {
      return ( 1.449259190449*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.190163509193)
	       +0.314134640646*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,1.784492748543)
	       +0.009060019811*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,5.525343742263)
	       );
    }

  /**
   *
   * Evaluation of the inverse Mellin transform for singlet parton
   * distributions with QED effects with a Gauss-Laguerre quadrature formula with 5 points.
   *
   */
  vector_d evolve_singlet_with_qed_5(const partondistribution & Delta, 
				     const partondistribution & s, const partondistribution & g, 
				     const partondistribution & phot, const partondistribution & lept, 
				     double Q, double x)
    {
      return ( 1.221725267*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.117581320)
	       +0.480277222*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,1.074562012)
	       +0.067748788*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,3.085937444)
	       +0.00268729149*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,6.414729734)
	       +0.0000152808657*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,11.807189490) 
	       );
    }

  /**
   *
   * Evaluation of the inverse Mellin transform for singlet parton
   * distributions with QED effects with a Gauss-Laguerre quadrature formula with 10 points.
   *
   */
  vector_d evolve_singlet_with_qed_10(const partondistribution & Delta, 
				      const partondistribution & s, const partondistribution & g, 
				      const partondistribution & phot, const partondistribution & lept, 
				      double Q, double x)
    {
      return ( 0.9244873392012*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.0601920631495879)
	       +0.5733510107257*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.5438675002946460)
	       +0.2180344120400*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,1.5229441054044437)
	       +0.04962104177493*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,3.02251337645157398)
	       +0.006487546684475*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,5.08490775009852397)
	       +0.0004566772720327*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,7.7774392315254451)
	       +0.00001560511295706*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,11.208130204348663)
	       +0.21721387415386E-6*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,15.561163332189350)
	       +0.87986819845464E-9*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,21.193892096301541)
	       +0.44587872910683E-12*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,29.024950340236226) 
	       );
    }

  /**
   *
   * Evaluation of the inverse Mellin transform for singlet parton
   * distributions with QED effects with a Gauss-Laguerre quadrature formula with 20 points.
   *
   */
  vector_d evolve_singlet_with_qed_20(const partondistribution & Delta, 
				      const partondistribution & s, const partondistribution & g, 
				      const partondistribution & phot, const partondistribution & lept, 
				      double Q, double x)
    {
      vector_d s1 = 0.6772865549*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.3046323928E-1)
	+0.5314565038*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.2744447158)+0.3267574654*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.7638875584)
	+0.1569492117*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.1501801498E1)+0.5862513107E-1*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.2492830145E1)
	+0.1692177602E-1*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.3743418041E1)+0.3742993659E-2*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.5262055854E1)
	+0.6277071891E-3*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.7059627736E1)+0.7873867962E-4*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.9149898312E1)
	+0.7263152301E-5*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.1155019829E2);
      vector_d t0 = s1+0.4822288327E-6*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.1428240369E2)
	+0.2242472166E-7*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.1737436698E2)+0.7051241583E-9*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.2086207519E2)
	+0.1431305611E-10*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.2479303989E2)+0.1761141529E-12*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.2923191016E2)
	+0.1201671758E-14*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.3427042893E2)+0.3978362024E-17*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.4004681579E2)
	+0.5135186731E-20*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.4678884639E2)+0.1708811393E-23*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.5493155562E2)
	+0.5182087428E-28*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.6558993199E2);
      return (t0);
    }

  /**
   *
   * Evaluation of the inverse Mellin transform for singlet parton
   * distributions with QED effects with a Gauss-Laguerre quadrature formula with 30 points.
   *
   */
  vector_d evolve_singlet_with_qed_30(const partondistribution & Delta, 
				      const partondistribution & s, const partondistribution & g, 
				      const partondistribution & phot, const partondistribution & lept, 
				      double Q, double x)
    {
      vector_d s1 = 0.4757378099*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.183612305)
	+0.559706235*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.2039219378E-1)+0.3435523138*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.5104942191)
	+0.2105975274*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.1001927927E1)+0.4812254553E-1*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.2484340278E1)
	+0.1094379419*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.1659265178E1)+0.1786435672E-1*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.3479499028E1)
	+0.5582649658E-2*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.4647636927E1)+0.3205546694E-3*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.7517487793E1)
	+0.1463547114E-2*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.5992248202E1)+0.5837483808E-4*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.9228249122E1)
	+0.1086703227E-5*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.1323021228E2)+0.8788573873E-5*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.1113026149E2)
	+0.1095110924E-6*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.1553590102E2)+0.891364555E-8*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.1805643521E2);
      vector_d t0 = s1+0.5798718026E-9*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.2080248162E2)
	+0.2977914698E-10*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.2378659288E2)+0.1189686103E-11*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.2702363844E2)
	+0.3633249153E-13*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.3053138327E2)+0.8304888219E-15*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.3433128161E2)
	+0.1384649582E-16*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.3844959272E2)+0.1630728095E-18*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.4291899667E2)
	+0.1302513451E-20*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.4778101845E2)+0.6691513912E-23*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.5308982661E2)
	+0.2058811994E-25*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.5891851875E2)+0.3431146295E-28*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.6537027557E2)
	+0.2665119224E-31*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.7260010093E2)+0.4879495176E-39*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.9064760683E2)
	+0.7543253454E-35*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.8086322182E2)+0.2219174496E-44*integration_contour::singlet_with_qed(Delta,s,g,phot,lept,Q,x,0.1032075007E3);
      return(t0);
    }

} // namespace pdf

