

/** @file anomalous.cc
 *
 * Implementation of anomalous dimensions
 *
 */

/*
 *  Copyright (C) 2002 Stefan Weinzierl
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <cmath>

#include "anomalous.h"
#include "constants_math.h"
#include "special_functions.h"

namespace pdf {

  /**
   *
   * The first coefficient \f$\gamma_0\f$ of the anomalous dimension 
   * of the quark non-singlet operator.
   *
   */
  complex_d anomalous_dimensions::anomalous_0_NS(const complex_d & N, int Nf)
    {
      return( 2.0*CF*( 4.0*S1(N)-3.0-2.0/N/(N+1.0)) );
    }

  /**
   *
   * The first coefficient \f$\gamma_0^{(qq)}\f$ of the anomalous dimension 
   * appearing in the anomalous dimension matrix for the quark singlet / gluon.
   *
   */
  complex_d anomalous_dimensions::anomalous_0_qq(const complex_d & N, int Nf)
    {
      return( 2.0*CF*( 4.0*S1(N)-3.0-2.0/N/(N+1.0)) );
    }

  /**
   *
   * The first coefficient \f$\gamma_0^{(qg)}\f$ of the anomalous dimension 
   * appearing in the anomalous dimension matrix for the quark singlet / gluon.
   *
   */
  complex_d anomalous_dimensions::anomalous_0_qg(const complex_d & N, int Nf)
    {
      return( -8.0*TR*Nf*( pow(N,2)+N+2.0 )/N/(N+1.0)/(N+2.0) );
    }

  /**
   *
   * The first coefficient \f$\gamma_0^{(gq)}\f$ of the anomalous dimension 
   * appearing in the anomalous dimension matrix for the quark singlet / gluon.
   *
   */
  complex_d anomalous_dimensions::anomalous_0_gq(const complex_d & N, int Nf)
    {
      return( -4.0*CF*( pow(N,2)+N+2.0 )/(N-1.0)/N/(N+1.0) );
    }

  /**
   *
   * The first coefficient \f$\gamma_0^{(gg)}\f$ of the anomalous dimension 
   * appearing in the anomalous dimension matrix for the quark singlet / gluon.
   *
   */
  complex_d anomalous_dimensions::anomalous_0_gg(const complex_d & N, int Nf)
    {
      return( 2.0*CA*( 4.0*S1(N)-11.0/3.0-4.0/N/(N-1.0)-4.0/(N+1.0)/(N+2.0) ) + 8.0/3.0*TR*Nf );
    }

  /**
   *
   * The second coefficient \f$\gamma_1\f$ of the anomalous dimension 
   * of the quark non-singlet operator.
   *
   * \f$ \eta = \pm 1\f$ selects the even or odd moments.
   *
   * Valence non-singlet distributions like
   * \f[
   *     (u-\bar{u}),
   *     (d - \bar{d}), ...
   * \f]
   * have \f$\eta=-1\f$, while
   * the singlet case and non-valence non-singlet combinations like
   * \f[
   *    (u+\bar{u}) - ( d + \bar{d} ),
   * \f]
   * \f[
   *    (u+\bar{u}) + ( d + \bar{d} ) - 2 ( s + \bar{s} ), ...
   * \f]
   * have \f$\eta=1\f$.
   * 
   */
  complex_d anomalous_dimensions::anomalous_1_NS(const complex_d & N, int eta, int Nf)
    {
      return( pow(CF,2)*( 16.0*S1(N)*(2.0*N+1.0)/pow(N*(N+1.0),2) + 16.0*(2.0*S1(N)-1.0/N/(N+1.0))
			  *(S2(N)-Sprim2(N/2.0,eta))
			  +24.0*S2(N)+64.0*Stilde(N,eta)-8.0*Sprim3(N/2.0,eta)
			  -3.0-8.0*(3.0*pow(N,3)+pow(N,2)-1.0)/pow(N*(N+1.0),3)-16.0*eta
			  *(2.0*pow(N,2)+2.0*N+1.0)/pow(N*(N+1.0),3) )
	      +CA*CF*(536.0/9.0*S1(N)-8.0*(2.0*S1(N)-1.0/N/(N+1.0))*(2.0*S2(N)-Sprim2(N/2.0,eta))
		      -88.0/3.0*S2(N)-32.0*Stilde(N,eta)+4.0*Sprim3(N/2.0,eta)-17.0/3.0
		      -4.0/9.0*(151.0*pow(N,4)+236.0*pow(N,3)+88.0*pow(N,2)+3.0*N+18.0)/pow(N*(N+1.0),3)
		      +8.0*eta*(2.0*pow(N,2)+2.0*N+1.0)/pow(N*(N+1.0),3) )
	      +CF*TR*Nf*( -160.0/9.0*S1(N)+32.0/3.0*S2(N)+4.0/3.0
			  +16.0/9.0*(11.0*pow(N,2)+5.0*N-3.0)/pow(N*(N+1.0),2) )  
	      ); 
    }

  /**
   *
   * The second coefficient \f$\gamma_1^{(qq)}\f$ of the anomalous dimension 
   * appearing in the anomalous dimension matrix for the quark singlet / gluon.
   *
   */
  complex_d anomalous_dimensions::anomalous_1_qq(const complex_d & N, int Nf)
    {
      return( anomalous_1_NS(N,1,Nf)-16.0*CF*TR*Nf*(5.0*pow(N,5)+32.0*pow(N,4)+49.0*pow(N,3)+38.0*pow(N,2)+28.0*N+8.0)
	      /(N-1.0)/pow(N*(N+1.0),3)/pow(N+2.0,2) );
    }

  /**
   *
   * The second coefficient \f$\gamma_1^{(qg)}\f$ of the anomalous dimension 
   * appearing in the anomalous dimension matrix for the quark singlet / gluon.
   *
   */
  complex_d anomalous_dimensions::anomalous_1_qg(const complex_d & N, int Nf)
    {
      return( -8.0*CA*TR*Nf*( (-2.0*pow(S1(N),2)+2.0*S2(N)-2.0*Sprim2(N/2.0,1))*(pow(N,2)+N+2.0)/N/(N+1.0)/(N+2.0)
			    +8.0*S1(N)*(2.0*N+3.0)/pow((N+1.0)*(N+2.0),2)
			    +2.0*(pow(N,9)+6.0*pow(N,8)+15.0*pow(N,7)+25.0*pow(N,6)+36.0*pow(N,5)+85.0*pow(N,4)
				+128.0*pow(N,3)+104.0*pow(N,2)+64.0*N+16.0)
			    /(N-1.0)/pow(N*(N+1.0)*(N+2.0),3) )
	      -8.0*CF*TR*Nf*( (2.0*pow(S1(N),2)-2.0*S2(N)+5.0)*(pow(N,2)+N+2.0)/N/(N+1.0)/(N+2.0)-4.0*S1(N)/pow(N,2)
			    +(11.0*pow(N,4)+26.0*pow(N,3)+15.0*pow(N,2)+8.0*N+4.0)/pow(N*(N+1.0),3)/(N+2.0) )  );
    }

  /**
   *
   * The second coefficient \f$\gamma_1^{(gq)}\f$ of the anomalous dimension 
   * appearing in the anomalous dimension matrix for the quark singlet / gluon.
   *
   */
  complex_d anomalous_dimensions::anomalous_1_gq(const complex_d & N, int Nf)
    {
      return( -4.0*pow(CF,2)*( (-2.0*pow(S1(N),2)+10.0*S1(N)-2.0*S2(N))*(pow(N,2)+N+2.0)/(N-1.0)/N/(N+1.0)
			     -4.0*S1(N)/pow(N+1.0,2)
			     -(12.0*pow(N,6)+30.0*pow(N,5)+43.0*pow(N,4)+28.0*pow(N,3)-pow(N,2)-12.0*N-4.0)
			     /(N-1.0)/pow(N*(N+1.0),3) )
	      -8.0*CF*CA*( (pow(S1(N),2)+S2(N)-Sprim2(N/2.0,1))*(pow(N,2)+N+2.0)/(N-1.0)/N/(N+1.0)
			 -S1(N)*(17.0*pow(N,4)+41.0*pow(N,2)-22.0*N-12.0)/3.0/pow((N-1.0)*N,2)/(N+1.0)
			 +(109.0*pow(N,9)+621.0*pow(N,8)+1400.0*pow(N,7)+1678.0*pow(N,6)+695.0*pow(N,5)
			   -1031.0*pow(N,4)-1304.0*pow(N,3)-152.0*pow(N,2)+432.0*N+144.0)
			 /9.0/pow((N-1.0)*(N+2.0),2)/pow(N*(N+1.0),3) )
	      -32.0/3.0*CF*TR*Nf*( (S1(N)-8.0/3.0)*(pow(N,2)+N+2.0)/(N-1.0)/N/(N+1.0)+1.0/pow(N+1.0,2) ) );
    }

  /**
   *
   * The second coefficient \f$\gamma_1^{(gg)}\f$ of the anomalous dimension 
   * appearing in the anomalous dimension matrix for the quark singlet / gluon.
   *
   */
  complex_d anomalous_dimensions::anomalous_1_gg(const complex_d & N, int Nf)
    {
      return( CA*TR*Nf*( -160.0/9.0*S1(N)+32.0/3.0+16.0/9.0*(38.0*pow(N,4)+76.0*pow(N,3)+94.0*pow(N,2)+56.0*N+12.0)
			 /(N-1.0)/pow(N*(N+1.0),2)/(N+2.0) )
	      +CF*TR*Nf*( 8.0+16.0*(2.0*pow(N,6)+4.0*pow(N,5)+pow(N,4)-10.0*pow(N,3)-5.0*pow(N,2)-4.0*N-4.0)
			  /(N-1.0)/pow(N*(N+1.0),3)/(N+2.0) ) 
	      +pow(CA,2)*( 536.0/9.0*S1(N)+64.0*S1(N)*(2.0*pow(N,5)+5.0*pow(N,4)+8.0*pow(N,3)+7.0*pow(N,2)-2.0*N-2.0)
			   /pow((N-1.0)*N*(N+1.0)*(N+2.0),2) -64.0/3.0
			   +32.0*Sprim2(N/2.0,1)*(pow(N,2)+N+1.0)/(N-1.0)/N/(N+1.0)/(N+2.0)
			   -16.0*S1(N)*Sprim2(N/2.0,1)+32.0*Stilde(N,1)-4.0*Sprim3(N/2.0,1)
			   -4.0/9.0*(457.0*pow(N,9)+2742.0*pow(N,8)+6040.0*pow(N,7)+6098.0*pow(N,6)+1567.0*pow(N,5)
				   -2344.0*pow(N,4)-1632.0*pow(N,3)+560.0*pow(N,2)+1488.0*N+576.0)
			   /pow(N-1.0,2)/pow(N*(N+1.0)*(N+2.0),3) ) );
    }

} // namespace pdf

