
/** @file anomalous.h
 *
 * Interface to anomalous dimensions
 *
 */

/*
 *  Copyright (C) 2002 Stefan Weinzierl
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __PDF_ANOMALOUS_H__
#define __PDF_ANOMALOUS_H__

#include "complex_d.h"

namespace pdf {

  /**
   *
   * Anomalous dimensions for the evolution of parton distributions.
   *
   * The anomalous dimensions are expanded in
   * \f$ a_s = \alpha_s / ( 4 \pi) \f$ as
   * \f[
   *    \gamma = \gamma_0 a_s + \gamma_1 a_s^2 + ...
   * \f]
   * and implemented in N-space.
   *
   * The formulae for the one-loop and two-loop anomalous dimensions 
   * can be found for example in
   *  E.G. Floratos, C. Kounnas and R. Lacaze, 
   *  Nucl. Phys. B192, (1981), 417.
   *
   */
  class anomalous_dimensions {

    // ctors

    // static functions
    public :
      static complex_d anomalous_0_NS(const complex_d & N, int Nf);
    
      static complex_d anomalous_0_qq(const complex_d & N, int Nf);
      static complex_d anomalous_0_qg(const complex_d & N, int Nf);
      static complex_d anomalous_0_gq(const complex_d & N, int Nf);
      static complex_d anomalous_0_gg(const complex_d & N, int Nf);

      static complex_d anomalous_1_NS(const complex_d & N, int eta, int Nf);

      static complex_d anomalous_1_qq(const complex_d & N, int Nf);
      static complex_d anomalous_1_qg(const complex_d & N, int Nf);
      static complex_d anomalous_1_gq(const complex_d & N, int Nf);
      static complex_d anomalous_1_gg(const complex_d & N, int Nf);

    // data members

  };

} // namespace pdf

#endif // ndef __PDF_ANOMALOUS_H__






