

/** @file coupling.cc
 *
 * Implementation of coupling
 *
 */

/*
 *  Copyright (C) 2002 Stefan Weinzierl
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <cmath>
#include <stdexcept>

#include "coupling.h"
#include "constants_math.h"

namespace pdf {

  /**
   *
   * The first coefficient \f$\beta_0\f$ of the beta function
   * \f[
   *    \mu^2 \frac{d}{d\mu^2} a_s = - \varepsilon a_s
   *       \mbox{} - \beta_0 a_s^2 - \beta_1 a_s^3 - \beta_2 a_s^4 - ...
   * \f]
   * with \f$ a_s = \alpha_s / ( 4 \pi) \f$ and
   * \f[
   *    \beta_0 = \frac{11}{3} C_A - \frac{4}{3} T_R N_f
   * \f]
   *
   */
  double alpha_strong::beta_0(int Nf)
    {
      return( (11*CA-4*TR*Nf)/3.0 );
    }

  /**
   *
   * The second coefficient \f$\beta_1\f$ of the beta function
   * \f[
   *    \mu^2 \frac{d}{d\mu^2} a_s = - \varepsilon a_s
   *       \mbox{} - \beta_0 a_s^2 - \beta_1 a_s^3 - \beta_2 a_s^4 - ...
   * \f]
   * with \f$ a_s = \alpha_s / ( 4 \pi) \f$ and
   * \f[
   *    \beta_1 = \frac{34}{3} C_A^2 - 4 \left( \frac{5}{3} C_A + C_F \right) T_R N_f
   * \f]
   *
   */
  double alpha_strong::beta_1(int Nf)
    {
      return( 34/3.0*pow(CA,2)-4*(5/3.0*CA+CF)*TR*Nf );
    }

  /**
   *
   * Returns the number of active flavours at the scale \f$Q\f$.
   *
   */ 
  int alpha_strong::get_active_flavours(double Q)
    {
      if ( Q < charm_threshold ) return 3;
      else if ( Q < bottom_threshold ) return 4;

      return 5;
    }

  /**
   *
   * Returns the appropriate \f$\Lambda_{QCD}\f$ corresponding 
   * to the scale \f$Q\f$.
   *
   */ 
  double alpha_strong::get_lambda_QCD(double Q)
    {
      if ( Q < charm_threshold ) return lambda_3;
      else if ( Q < bottom_threshold ) return lambda_4;

      return lambda_5;
    }

  /**
   *
   * Returns the appropriate \f$\Lambda_{QCD}\f$ corresponding 
   * to \f$N_f\f$ active flavours.
   *
   */ 
  double alpha_strong::get_lambda_QCD(int Nf)
    {
      if ( Nf == 3) return lambda_3;
      else if ( Nf == 4) return lambda_4;

      return lambda_5;
    }

  /**
   *
   * Returns the value of \f$ a_s = \frac{\alpha_s}{4\pi}\f$
   * at the scale \f$Q\f$ with \f$N_f\f$ active flavours.
   *
   * The parameter \f$N_f\f$ ensures that the correct
   * formula is used close to flavour thresholds.
   *
   */
  double alpha_strong::get_value(double Q, int Nf)
    {
      // constants
      double beta0 = alpha_strong::beta_0(Nf);
      double beta1 = alpha_strong::beta_1(Nf);

      // define L
      double lambda = get_lambda_QCD(Nf);
      double L  = 2.0*log(Q/lambda);  


      if ( method == truncate_in_one_over_L)
	{
	  return 1/beta0/L*(1-beta1/pow(beta0,2)*log(L)/L);
	}
      else if ( method == exact )
	{
	  double a_low = 0.0001;
	  double a_high = 1.0/beta0/L;

	  return root_safeI(a_low,a_high,Q,Nf);
	}

      // default, should not occur
      return 0.0;
    }

  /**
   *
   * Evaluates
   * \f[
   *    F(Q,a_s) = \frac{1}{a_s} 
   *     \mbox{} - \frac{\beta_1}{\beta_0} \ln \left( \beta_1 + \frac{\beta_0}{a_s} \right)
   *     \mbox{} - \beta_0 L
   * \f]
   * with \f$ L = \ln \frac{Q^2}{\Lambda^2_{QCD}}\f$.
   *
   */
  double alpha_strong::get_F(double Q, double a_s, int Nf)
    {
      // constants
      double beta0 = alpha_strong::beta_0(Nf);
      double beta1 = alpha_strong::beta_1(Nf);

      // define L
      double lambda = get_lambda_QCD(Nf);
      double L  = 2.0*log(Q/lambda);  

      return 1.0/a_s - beta1/beta0*log(beta1+beta0/a_s) - beta0*L;
    }

  /**
   *
   * Evaluates
   * \f[
   *    \frac{d}{da_s} F(Q,a_s) = - \frac{\beta_0}{a_s^2} 
   *     \frac{1}{(\beta_0+\beta_1 a_s)}.
   * \f]
   *
   */
  double alpha_strong::get_dF(double Q, double a_s, int Nf)
    {
      // constants
      double beta0 = alpha_strong::beta_0(Nf);
      double beta1 = alpha_strong::beta_1(Nf);

      return - beta0/pow(a_s,2)/(beta0+beta1*a_s);
    }

  /**
   *
   * Numerical solution of the equation 
   * \f[
   *    \frac{1}{a_s} 
   *     \mbox{} - \frac{\beta_1}{\beta_0} \ln \left( \beta_1 + \frac{\beta_0}{a_s} \right)
   *     \mbox{} - \beta_0 L = 0
   * \f]
   * for \f$a_s\f$.
   *
   */
  double alpha_strong::root_safeI(double a, double b, double Q, int Nf) 
    {
      double f,df,flow,fhigh;
      double x1,x2,xlow,xhigh,rts,dxold,dx;
      int j;
      double temp;

      // initial guess and brackets
      x1=a;
      x2=b; 

      flow = get_F(Q,x1,Nf);
      fhigh= get_F(Q,x2,Nf);

      if (flow*fhigh >= 0.0)
	{ 
	  throw(std::runtime_error("Root must be bracketed "));
	}
      if (flow < 0.0)  // orient search so that f(x1)<0
	{ 
	  xlow = x1;
	  xhigh= x2;
	}
      else
	{
	  xhigh = x1;
	  xlow  = x2;
	}
      rts=0.5*(x1+x2); // initial guess for root
      dxold=fabs(x2-x1);
      dx=dxold;

      f = get_F(Q,rts,Nf);
      df = get_dF(Q,rts,Nf);

      for (j=1;j<=100;j++)
	{
	  if ( (((rts-xhigh)*df-f)*((rts-xlow)*df-f) >= 0.0) // bisect if Newton out of range
	       || (fabs(2.0*f) > fabs(dxold*df)) )            // or not decreasing fast enough
	    {
	      dxold=dx;
	      dx=0.5*(xhigh-xlow);
	      rts=xlow+dx;
	      if (xlow == rts) return rts;    //  change in root negligible
	    }
	  else
	    {
	      dxold=dx;
	      dx=f/df;
	      temp=rts;
	      rts -= dx;
	      if (temp == rts) return rts;   //  change in root negligible
	    }
	  if (fabs(dx) < 1E-8) return rts;   // convergence criterion
	  f = get_F(Q,rts,Nf);
	  df= get_dF(Q,rts,Nf);
	  if (f < 0.0 ) 
	    xlow = rts;
	  else
	    xhigh = rts;
	}

      throw(std::runtime_error("Maximum number of iterations exceeded in root_safe "));

    }

  // static data members

  double alpha_strong::lambda_3 = 0.374;
  double alpha_strong::lambda_4 = 0.327;
  double alpha_strong::lambda_5 = 0.226;

  double alpha_strong::charm_threshold  = 1.3;
  double alpha_strong::bottom_threshold = 4.7;

  //  int alpha_strong::method = alpha_strong::exact;
  int alpha_strong::method = alpha_strong::truncate_in_one_over_L;

} // namespace pdf

