
/** @file coupling.h
 *
 * Interface to coupling 
 *
 */

/*
 *  Copyright (C) 2002 Stefan Weinzierl
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __PDF_COUPLING_H__
#define __PDF_COUPLING_H__

namespace pdf {

  /**
   *
   * The strong coupling constant.
   *
   * The class works with the quantity 
   *  \f[
   *     a_s = \frac{\alpha_s}{4\pi}
   * \f].
   * 
   * If alpha_strong::method == alpha_strong::exact, the value of \f$a_s\f$
   * is obtained by solving numerically the equation
   * \f[
   *    \frac{1}{a_s} 
   *     \mbox{} - \frac{\beta_1}{\beta_0} \ln \left( \beta_1 + \frac{\beta_0}{a_s} \right)
   *     \mbox{} - \beta_0 L = 0
   * \f]
   * where \f$ L = \ln \frac{Q^2}{\Lambda^2_{QCD}}\f$.
   *
   * If alpha_strong::method == alpha_strong::truncate_in_one_over_L, the value of \f$a_s\f$
   * is given by the formula
   * \f[
   *     a_s = \frac{1}{\beta_0 L} \left( 1 - \frac{\beta_1}{\beta_0^2} \frac{\ln L}{L} \right).
   * \f]
   *
   */
  class alpha_strong {

    // ctors

    // static functions
    public :
      static double beta_0(int Nf);
      static double beta_1(int Nf);

      static int get_active_flavours(double Q);
      static double get_lambda_QCD(double Q);
      static double get_lambda_QCD(int Nf);
      static double get_value(double Q, int Nf);

    private :
      static double get_F(double Q, double a_s, int Nf);
      static double get_dF(double Q, double a_s, int Nf);
      static double root_safeI(double a, double b, double Q, int Nf);

    // data members
    public :
      static double lambda_3;
      static double lambda_4;
      static double lambda_5;

      static double charm_threshold;
      static double bottom_threshold;

      static int method;

    // keywords
      enum {
	exact,
        truncate_in_one_over_L
      };

  };

} // namespace pdf

#endif // ndef __PDF_COUPLING_H__

