

/** @file coupling_qed.cc
 *
 * Implementation of coupling_qed
 *
 */

/*
 *  Copyright (C) 2002 Stefan Weinzierl
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <cmath>
#include <stdexcept>

#include "coupling_qed.h"
#include "constants_math.h"

namespace pdf {

  /**
   *
   * The first coefficient \f$\beta_0\f$ of the QED beta function
   * \f[
   *    \mu^2 \frac{d}{d\mu^2} a_{fs} = - \varepsilon a_{fs}
   *       \mbox{} - \beta_0 a_{fs}^2 - \beta_1 a_{fs}^3 - \beta_2 a_{fs}^4 - ...
   * \f]
   * with \f$ a_{fs} = \alpha_{fs} / ( 4 \pi) \f$ and
   * \f[
   *    \beta_0 =  - \frac{4}{3} \sum\limits_{i=e,\mu,\tau,u,d,s,c,b} N_c(i) Q_i^2
   * \f]
   *
   */
  double alpha_finestruc::beta_0(int Nf_down, int Nf_up, int Nl)
    {
      return( -4.0/3.0*( Nl*pow(Q_electron,2) 
			 + Nc*( Nf_down*pow(Q_down,2) + Nf_up*pow(Q_up,2) ) 
			 ) );
    }

  /**
   *
   * Returns the number of active flavours at the scale \f$Q\f$.
   *
   */ 
  int alpha_finestruc::get_active_flavours(double Q)
    {
      if ( Q < charm_threshold ) return 3;
      else if ( Q < bottom_threshold ) return 4;

      return 5;
    }

  /**
   *
   * Returns the number of active leptons at the scale \f$Q\f$.
   *
   */ 
  int alpha_finestruc::get_active_leptons(double Q)
    {
      if ( Q < tau_threshold ) return 2;

      return 3;
    }

  /**
   *
   * Returns the value of \f$ a_{fs} = \frac{\alpha_{fs}}{4\pi}\f$
   * at the scale \f$Q\f$ with \f$N_f=N_{f,down}+N_{f,up}\f$ active flavours and
   * \f$N_l\f$ active leptons.
   *
   * The top quark is neglected. For a detailed discussion see
   * S. Eidelman and F. Jegerlehner, hep-ph/9502298
   *
   */
  double alpha_finestruc::get_value(double Q, int Nf_down, int Nf_up, int Nl)
    {
      int Nf = Nf_down + Nf_up;

      // constants
      double beta0 = alpha_finestruc::beta_0(Nf_down,Nf_up,Nl);

      if ( Nf == 3) // below charm
	{
	  double L  = 2.0*log(Q/charm_threshold);  
	  return value_at_charm/( 1.0 + beta0*value_at_charm * L );
	}
      else if ( Nf == 4) 
	{
	  if (Nl == 2) // below tau 
	    {
	      double L  = 2.0*log(Q/charm_threshold);  
	      return value_at_charm/( 1.0 + beta0*value_at_charm * L );
	    }
	  else // above tau
	    {
	      double L  = 2.0*log(Q/tau_threshold);  
	      return value_at_tau/( 1.0 + beta0*value_at_tau * L );
	    }
	}
      else if ( Nf == 5) // above bottom
	{
	  double L  = 2.0*log(Q/bottom_threshold);  
	  return value_at_bottom/( 1.0 + beta0*value_at_bottom * L );
	}

      // default, should not occur
      return 0.0;
    }

  // static data members

  double alpha_finestruc::value_at_charm  = 0.0005949448010; 
  double alpha_finestruc::value_at_tau    = 0.0005965224393; 
  double alpha_finestruc::value_at_bottom = 0.0006024255586; 

  double alpha_finestruc::charm_threshold  = 1.3;
  double alpha_finestruc::tau_threshold    = 1.777;
  double alpha_finestruc::bottom_threshold = 4.7;

} // namespace pdf

