

/** @file evolutionkernel.cc
 *
 * Implementation of evolutionkernel 
 *
 */

/*
 *  Copyright (C) 2002 Stefan Weinzierl
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <cmath>

#include "complex_d.h"
#include "matrix_c.h"
#include "special_functions.h"

#include "evolutionkernel.h"
#include "coupling.h"
#include "coupling_qed.h"
#include "constants_math.h"
#include "anomalous.h"

namespace pdf {

  /**
   *
   * Returns the evolutionkernel
   * \f[
   *    E^z(Q^2,Q_0^2) 
   * \f]
   * for the non-singlet case. 
   *
   * If the evolution goes over flavour threshold, the evolution is split
   * into separate pieces. For a fixed number of active flavour, the kernel
   * is obtained from evolutionkernel::nonsinglet_fixed_flavour.
   *
   */
  complex_d evolutionkernel::nonsinglet(const complex_d & z, double Q, double Q0, int eta)
    {
      double mc = alpha_strong::charm_threshold;
      double mb = alpha_strong::bottom_threshold;

      if ( Q0 < mc)
	{
	  if ( Q < mc )
	    { 
	      return evolutionkernel::nonsinglet_fixed_flavour(z,Q,Q0,3,eta);
	    }
	  else if ( Q < mb )
	    { 
	      return evolutionkernel::nonsinglet_fixed_flavour(z,Q,mc,4,eta)
		* evolutionkernel::nonsinglet_fixed_flavour(z,mc,Q0,3,eta);
	    }
	  else
	    { 
	      return evolutionkernel::nonsinglet_fixed_flavour(z,Q,mb,5,eta)
		* evolutionkernel::nonsinglet_fixed_flavour(z,mb,mc,4,eta)
		* evolutionkernel::nonsinglet_fixed_flavour(z,mc,Q0,3,eta);
	    }
	}
      else if ( Q0 < mb)
	{
	  if ( Q < mb )
	    { 
	      return evolutionkernel::nonsinglet_fixed_flavour(z,Q,Q0,4,eta);
	    }
	  else
	    { 
	      return evolutionkernel::nonsinglet_fixed_flavour(z,Q,mb,5,eta)
		* evolutionkernel::nonsinglet_fixed_flavour(z,mb,Q0,4,eta);
	    }
	}
      else
	{
	  return evolutionkernel::nonsinglet_fixed_flavour(z,Q,Q0,5,eta);
	}

      // default, should not occur
      return 0.0;

    }

  /**
   *
   * Returns the evolutionkernel
   * \f[
   *    E^z(Q^2,Q_0^2) 
   * \f]
   * for the singlet case. 
   *
   * If the evolution goes over flavour threshold, the evolution is split
   * into separate pieces. For a fixed number of active flavour, the kernel
   * is obtained from evolutionkernel::singlet_fixed_flavour.
   *
   */
  matrix_c evolutionkernel::singlet(const complex_d & z, double Q, double Q0)
    {
      double mc = alpha_strong::charm_threshold;
      double mb = alpha_strong::bottom_threshold;

      if ( Q0 < mc)
	{
	  if ( Q < mc )
	    { 
	      return evolutionkernel::singlet_fixed_flavour(z,Q,Q0,3);
	    }
	  else if ( Q < mb )
	    { 
	      return evolutionkernel::singlet_fixed_flavour(z,Q,mc,4)
		* evolutionkernel::singlet_fixed_flavour(z,mc,Q0,3);
	    }
	  else
	    { 
	      return evolutionkernel::singlet_fixed_flavour(z,Q,mb,5)
		* evolutionkernel::singlet_fixed_flavour(z,mb,mc,4)
		* evolutionkernel::singlet_fixed_flavour(z,mc,Q0,3);
	    }
	}
      else if ( Q0 < mb)
	{
	  if ( Q < mb )
	    { 
	      return evolutionkernel::singlet_fixed_flavour(z,Q,Q0,4);
	    }
	  else
	    { 
	      return evolutionkernel::singlet_fixed_flavour(z,Q,mb,5)
		* evolutionkernel::singlet_fixed_flavour(z,mb,Q0,4);
	    }
	}
      else
	{
	  return evolutionkernel::singlet_fixed_flavour(z,Q,Q0,5);
	}

      // default, should not occur
      return matrix_c(0.0,2,2);
    }

  /**
   *
   * The non-singlet evolution kernel for a fixed number of active flavours.
   *
   * Depending on the value of evolutionkernel::method the kernel of the
   * corresponding truncation method is returned.
   *
   */
  complex_d evolutionkernel::nonsinglet_fixed_flavour(const complex_d & z, double Q1, double Q0, int Nf, int eta)
    {
      double temp;

      // constants
      double beta0 = alpha_strong::beta_0(Nf);
      double beta1 = alpha_strong::beta_1(Nf);

      double lambda = alpha_strong::get_lambda_QCD(Nf);

      // define L's
      double L  = 2.0*log(Q1/lambda);  
      double L0 = 2.0*log(Q0/lambda); 

      // alpha_s at the different scales
      // calculate a_s/4/Pi
      double strong1 = alpha_strong::get_value(Q1,Nf);
      double strong0 = alpha_strong::get_value(Q0,Nf);
 
      // define kernel
      complex_d P0    = -0.5*anomalous_dimensions::anomalous_0_NS(z,Nf);
      complex_d P1    = -0.5*anomalous_dimensions::anomalous_1_NS(z,eta,Nf);
      complex_d P1bar = P1-beta1/beta0*P0;

      if ( method == N_space)
	{
	  return ( pow(complex_d(strong1/strong0,0.0),-P0/beta0)
		   *(1.0-(strong1-strong0)*P1bar/beta0) ); 
	}
      else if ( method == x_space )
	{
	  return( pow(complex_d(strong1/strong0,0.0),-P0/beta0)
		  *pow((beta0+beta1*strong1)/(beta0+beta1*strong0),-P1bar/beta1) );
	}
      else if ( method == x_space_truncate_in_one_over_L )
	{
	  return ( pow(complex_d(L0/L,0.0),-P0/beta0)
		   *exp( P1bar/pow(beta0,2)*(1/L0-1/L)
			 -beta1/pow(beta0,3)*P0*(log(L0)/L0-log(L)/L)
			 -beta1/2.0/pow(beta0,4)
			 *P1*( (1+2*log(L0))/pow(L0,2) - (1+2*log(L))/pow(L,2))
			 +pow(beta1,2)/27.0/pow(beta0,6)
			 *P1*((2.0+6*log(L0)+9*pow(log(L0),2))/pow(L0,3)
			      -(2.0+6*log(L)+9*pow(log(L),2))/pow(L,3) ) ) ); 
	}

      // default should not occur
      return 0.0;
    }

  /**
   *
   * The singlet evolution kernel for a fixed number of active flavours.
   *
   * Depending on the value of evolutionkernel::method the kernel of the
   * corresponding truncation method is returned.
   *
   * For the methods x_space and x_space_truncate_in_one_over_L no closed
   * forms for the kernel are known and the kernel is obtained in these
   * cases by iteration of evolution in a small intervall.
   * This slows down performance considerably.
   *
   * The method N_space is therefore the recommended one.
   *
   */
  matrix_c evolutionkernel::singlet_fixed_flavour(const complex_d & z, double Q1, double Q0, int Nf)
    {
      // constants
      double beta0 = alpha_strong::beta_0(Nf);
      double beta1 = alpha_strong::beta_1(Nf);

      double lambda = alpha_strong::get_lambda_QCD(Nf);

      // get the anomalous dimensions
      complex_d gamma_0_qq = anomalous_dimensions::anomalous_0_qq(z,Nf);
      complex_d gamma_0_qg = anomalous_dimensions::anomalous_0_qg(z,Nf);
      complex_d gamma_0_gq = anomalous_dimensions::anomalous_0_gq(z,Nf);
      complex_d gamma_0_gg = anomalous_dimensions::anomalous_0_gg(z,Nf);

      complex_d M0[4] = { gamma_0_qq, gamma_0_qg,
			  gamma_0_gq, gamma_0_gg };
      matrix_c P0 = -0.5*matrix_c(M0,2,2);

      complex_d M1[4] = { anomalous_dimensions::anomalous_1_qq(z,Nf), 
			  anomalous_dimensions::anomalous_1_qg(z,Nf),
			  anomalous_dimensions::anomalous_1_gq(z,Nf), 
			  anomalous_dimensions::anomalous_1_gg(z,Nf) };
      matrix_c P1 = -0.5*matrix_c(M1,2,2);

      matrix_c P1bar = P1 - (beta1/beta0) * P0;

      complex_d MUnit[4] = { complex_d(1.0,0.0),complex_d(0.0,0.0),
			     complex_d(0.0,0.0),complex_d(1.0,0.0) };
      matrix_c Unit = matrix_c(MUnit,2,2);

      // define eigenvalues of one-loop anomalous dimension matrix
      complex_d diskriminante = pow( pow( gamma_0_gg-gamma_0_qq ,2) + 4.0*gamma_0_qg*gamma_0_gq , 0.5);
      // include a factor -0.5 from   P = - 1/2 gamma 
      // and change lambda_p <-> lambda_m 
      complex_d lambda_p = -0.5*0.5*(gamma_0_qq+gamma_0_gg-diskriminante);
      complex_d lambda_m = -0.5*0.5*(gamma_0_qq+gamma_0_gg+diskriminante);

      // define projection operators on eigenvalues
      matrix_c Pplus  =  1.0/(lambda_p-lambda_m) * ( P0 - lambda_m * Unit);
      matrix_c Pminus = -1.0/(lambda_p-lambda_m) * ( P0 - lambda_p * Unit);

      // define L's
      double Lup = 2.0*log(Q1/lambda);  
      double Llow = 2.0*log(Q0/lambda); 

      // alpha_s at the different scales
      // calculate a_s/4/Pi
      double strongup = alpha_strong::get_value(Q1,Nf);
      double stronglow = alpha_strong::get_value(Q0,Nf);

      matrix_c kernel = Unit;

      if ( method == N_space)
	{
	  double strong1 = strongup;
	  double strong0 = stronglow;

	  kernel = ( pow(strong1/strong0,-lambda_m/beta0)
		     *(Pminus
		       -(strong1-strong0)/beta0*Pminus*P1bar*Pminus
		       -((strong1*pow(strong1/strong0,(lambda_m-lambda_p)/beta0)-strong0)
			 /(beta0-lambda_p+lambda_m))*Pminus*P1bar*Pplus) 
		     + pow(strong1/strong0,-lambda_p/beta0)
		     *(Pplus 
		       -(strong1-strong0)/beta0*Pplus*P1bar*Pplus
		       -((strong1*pow(strong1/strong0,(lambda_p-lambda_m)/beta0)-strong0)
			 /(beta0-lambda_m+lambda_p))*Pplus*P1bar*Pminus) 
		     ) * kernel;
	}
      else if ( method == x_space )
	{
	  for (int iteration=1; iteration <= number_of_iterations; iteration++)
	    {
	      double strong1 = stronglow +  iteration   *(strongup-stronglow)/number_of_iterations;
	      double strong0 = stronglow + (iteration-1)*(strongup-stronglow)/number_of_iterations;

	      kernel = ( pow(strong1/strong0,-lambda_m/beta0)
			 *(Pminus
			   -log((beta0+beta1*strong1)/(beta0+beta1*strong0))/beta1*Pminus*P1bar*Pminus
			   -Pminus*P1bar*Pplus/(beta0-lambda_p+lambda_m)*
			   (strong1*pow(strong1/strong0,(lambda_m-lambda_p)/beta0)
			    *HyperGeometric(complex_d(1.0,0.0),1.0-(lambda_p-lambda_m)/beta0,
					    2.0-(lambda_p-lambda_m)/beta0,-beta1/beta0*strong1)
			    -strong0*HyperGeometric(complex_d(1.0,0.0),1.0-(lambda_p-lambda_m)/beta0,
						    2.0-(lambda_p-lambda_m)/beta0,-beta1/beta0*strong0) 
			    ) ) 
			 +  pow(strong1/strong0,-lambda_p/beta0)
			 *(Pplus
			   -log((beta0+beta1*strong1)/(beta0+beta1*strong0))/beta1*Pplus*P1bar*Pplus
			   -Pplus*P1bar*Pminus/(beta0-lambda_m+lambda_p)*
			   (strong1*pow(strong1/strong0,(lambda_p-lambda_m)/beta0)
			    *HyperGeometric(complex_d(1.0,0.0),1.0-(lambda_m-lambda_p)/beta0,
					    2.0-(lambda_m-lambda_p)/beta0,-beta1/beta0*strong1)
			    -strong0*HyperGeometric(complex_d(1.0,0.0),1.0-(lambda_m-lambda_p)/beta0,
						    2.0-(lambda_m-lambda_p)/beta0,-beta1/beta0*strong0) 
			    ) ) 
			 ) * kernel;
	    } // end iteration
	}
      else if ( method == x_space_truncate_in_one_over_L )
	{
	  for (int iteration=1; iteration <= number_of_iterations; iteration++)
	    {
	      complex_d del;

	      double L  = Llow +  iteration   *(Lup-Llow)/number_of_iterations;
	      double L0 = Llow + (iteration-1)*(Lup-Llow)/number_of_iterations;

	      del = (lambda_m-lambda_p)/beta0;

	      complex_d nondiagmp = 
		1.0/beta0/(beta0+lambda_m-lambda_p)*(1.0/L0-1.0/L*pow(L0/L,del))
		-2.0*beta1/pow(beta0,3)/(2.0*beta0+lambda_m-lambda_p)*(log(L0)/pow(L0,2)-log(L)/pow(L,2)*pow(L0/L,del))
		-2.0*beta1/pow(beta0,2)/pow(2.0*beta0+lambda_m-lambda_p,2)*(1.0/pow(L0,2)-1.0/pow(L,2)*pow(L0/L,del))
		+pow(beta1,2)/pow(beta0,5)/(3.0*beta0+lambda_m-lambda_p)*(pow(log(L0),2)/pow(L0,3)-pow(log(L),2)/pow(L,3)*pow(L0/L,del))
		+2.0*pow(beta1,2)/pow(beta0,4)/pow(3.0*beta0+lambda_m-lambda_p,2)*(log(L0)/pow(L0,3)-log(L)/pow(L,3)*pow(L0/L,del))
		+2.0*pow(beta1,2)/pow(beta0,3)/pow(3.0*beta0+lambda_m-lambda_p,3)*(1.0/pow(L0,3)-1.0/pow(L,3)*pow(L0/L,del));
 
	      del = (lambda_p-lambda_m)/beta0;

	      complex_d nondiagpm = 
		1.0/beta0/(beta0+lambda_p-lambda_m)*(1.0/L0-1.0/L*pow(L0/L,del))
		-2.0*beta1/pow(beta0,3)/(2.0*beta0+lambda_p-lambda_m)*(log(L0)/pow(L0,2)-log(L)/pow(L,2)*pow(L0/L,del))
		-2.0*beta1/pow(beta0,2)/pow(2.0*beta0+lambda_p-lambda_m,2)*(1.0/pow(L0,2)-1.0/pow(L,2)*pow(L0/L,del))
		+pow(beta1,2)/pow(beta0,5)/(3.0*beta0+lambda_p-lambda_m)*(pow(log(L0),2)/pow(L0,3)-pow(log(L),2)/pow(L,3)*pow(L0/L,del))
		+2.0*pow(beta1,2)/pow(beta0,4)/pow(3.0*beta0+lambda_p-lambda_m,2)*(log(L0)/pow(L0,3)-log(L)/pow(L,3)*pow(L0/L,del))
		+2.0*pow(beta1,2)/pow(beta0,3)/pow(3.0*beta0+lambda_p-lambda_m,3)*(1.0/pow(L0,3)-1.0/pow(L,3)*pow(L0/L,del));


	      kernel = 
		( pow(L0/L,-lambda_m/beta0)
		  *( Pminus 
		     + Pminus * ( 
				 ( 1/pow(beta0,2)*(1/L0-1/L)*P1bar
				   -beta1/pow(beta0,3)*(log(L0)/L0-log(L)/L)*P0
				   -beta1/2.0/pow(beta0,4)*((1+2*log(L0))/pow(L0,2)
							    -(1+2*log(L))/pow(L,2))*P1
				   +pow(beta1,2)/27.0/pow(beta0,6)
				   *((2+6*log(L0)+9*pow(log(L0),2))/pow(L0,3)
				     -(2+6*log(L)+9*pow(log(L),2))/pow(L,3))*P1  
				   ) 
				 * Pminus
				 )
		     + Pminus * nondiagmp*P1 * Pplus  
		     )

		  + pow(L0/L,-lambda_p/beta0)
		  *( Pplus 
		     + Pplus * (
				( 1/pow(beta0,2)*(1/L0-1/L)*P1bar
				  -beta1/pow(beta0,3)*(log(L0)/L0-log(L)/L)*P0
				  -beta1/2.0/pow(beta0,4)*((1+2*log(L0))/pow(L0,2)
							   -(1+2*log(L))/pow(L,2))*P1
				  +pow(beta1,2)/27.0/pow(beta0,6)*
				  ((2+6*log(L0)+9*pow(log(L0),2))/pow(L0,3)
				   -(2+6*log(L)+9*pow(log(L),2))/pow(L,3))*P1  
				  )
				* Pplus
				)   
		     + Pplus * nondiagpm*P1 * Pminus 
		     )
	      
		  ) * kernel; 
	    } // end iteration
	}

      return( kernel );
    }

  /**
   *
   * Returns the evolutionkernel
   * \f[
   *    E^z(Q^2,Q_0^2) 
   * \f]
   * for the non-singlet case with the inclusion of QED effects
   *
   * If the evolution goes over flavour threshold, the evolution is split
   * into separate pieces. For a fixed number of active flavour, the kernel
   * is obtained from evolutionkernel::nonsinglet_with_qed_fixed_flavour.
   *
   */
  complex_d evolutionkernel::nonsinglet_with_qed(const complex_d & z, double Q, double Q0, int eta, double electric_charge)
    {
      //  mc < mtau < mb
      double mc   = alpha_strong::charm_threshold;
      double mtau = alpha_finestruc::tau_threshold;
      double mb   = alpha_strong::bottom_threshold;

      if ( Q0 < mc)
	{
	  if ( Q < mc )
	    { 
	      return evolutionkernel::nonsinglet_with_qed_fixed_flavour(z,Q,Q0,2,1,2,eta,electric_charge);
	    }
	  else if ( Q < mtau )
	    { 
	      return evolutionkernel::nonsinglet_with_qed_fixed_flavour(z,Q,mc,2,2,2,eta,electric_charge)
		* evolutionkernel::nonsinglet_with_qed_fixed_flavour(z,mc,Q0,2,1,2,eta,electric_charge);
	    }
	  else if ( Q < mb )
	    { 
	      return evolutionkernel::nonsinglet_with_qed_fixed_flavour(z,Q,mtau,2,2,3,eta,electric_charge)
		* evolutionkernel::nonsinglet_with_qed_fixed_flavour(z,mtau,mc,2,2,2,eta,electric_charge)
		* evolutionkernel::nonsinglet_with_qed_fixed_flavour(z,mc,Q0,2,1,2,eta,electric_charge);
	    }
	  else
	    { 
	      return evolutionkernel::nonsinglet_with_qed_fixed_flavour(z,Q,mb,3,2,3,eta,electric_charge)
		* evolutionkernel::nonsinglet_with_qed_fixed_flavour(z,mb,mtau,2,2,3,eta,electric_charge)
		* evolutionkernel::nonsinglet_with_qed_fixed_flavour(z,mtau,mc,2,2,2,eta,electric_charge)
		* evolutionkernel::nonsinglet_with_qed_fixed_flavour(z,mc,Q0,2,1,2,eta,electric_charge);
	    }
	}
      else if ( Q0 < mtau)
	{
	  if ( Q < mtau )
	    { 
	      return evolutionkernel::nonsinglet_with_qed_fixed_flavour(z,Q,Q0,2,2,2,eta,electric_charge);
	    }
	  else if ( Q < mb )
	    { 
	      return evolutionkernel::nonsinglet_with_qed_fixed_flavour(z,Q,mtau,2,2,3,eta,electric_charge)
		* evolutionkernel::nonsinglet_with_qed_fixed_flavour(z,mtau,Q0,2,2,2,eta,electric_charge);
	    }
	  else
	    { 
	      return evolutionkernel::nonsinglet_with_qed_fixed_flavour(z,Q,mb,3,2,3,eta,electric_charge)
		* evolutionkernel::nonsinglet_with_qed_fixed_flavour(z,mb,mtau,2,2,3,eta,electric_charge)
		* evolutionkernel::nonsinglet_with_qed_fixed_flavour(z,mtau,Q0,2,2,2,eta,electric_charge);
	    }
	}
      else if ( Q0 < mb)
	{
	  if ( Q < mb )
	    { 
	      return evolutionkernel::nonsinglet_with_qed_fixed_flavour(z,Q,Q0,2,2,3,eta,electric_charge);
	    }
	  else
	    { 
	      return evolutionkernel::nonsinglet_with_qed_fixed_flavour(z,Q,mb,3,2,3,eta,electric_charge)
		* evolutionkernel::nonsinglet_with_qed_fixed_flavour(z,mb,Q0,2,2,3,eta,electric_charge);
	    }
	}
      else
	{
	  return evolutionkernel::nonsinglet_with_qed_fixed_flavour(z,Q,Q0,3,2,3,eta,electric_charge);
	}

      // default, should not occur
      return 0.0;

    }

  /**
   *
   * The non-singlet evolution kernel for a fixed number of active flavours and with the inclusion of QED effects.
   *
   */
  complex_d evolutionkernel::nonsinglet_with_qed_fixed_flavour(const complex_d & z, double Q1, double Q0, int Nf_down, int Nf_up, int Nl, int eta, double electric_charge)
    {
      double temp;

      int Nf = Nf_down + Nf_up;

      // constants
      double beta0 = alpha_strong::beta_0(Nf);
      double beta1 = alpha_strong::beta_1(Nf);

      double beta0_qed = alpha_finestruc::beta_0(Nf_down, Nf_up, Nl);

      double lambda = alpha_strong::get_lambda_QCD(Nf);

      // define L's
      double L  = 2.0*log(Q1/lambda);  
      double L0 = 2.0*log(Q0/lambda); 

      // alpha_s at the different scales
      // calculate a_s/4/Pi
      double strong1 = alpha_strong::get_value(Q1,Nf);
      double strong0 = alpha_strong::get_value(Q0,Nf);

      double finestruc1 = alpha_finestruc::get_value(Q1,Nf_down,Nf_up,Nl);
      double finestruc0 = alpha_finestruc::get_value(Q0,Nf_down,Nf_up,Nl);
 
      // define kernel
      complex_d P0    = -0.5*anomalous_dimensions::anomalous_0_NS(z,Nf);
      complex_d P1    = -0.5*anomalous_dimensions::anomalous_1_NS(z,eta,Nf);
      complex_d P1bar = P1-beta1/beta0*P0;

      // divide out CF contained in anomalous_0_NS
      complex_d P0_qed = -0.5 * pow(electric_charge,2) / CF *anomalous_dimensions::anomalous_0_NS(z,Nf);

      return ( pow(complex_d(L0/L,0.0),-P0/beta0)
	       *exp( P1bar/pow(beta0,2)*(1/L0-1/L)
		     -beta1/pow(beta0,3)*P0*(log(L0)/L0-log(L)/L)
		     -beta1/2.0/pow(beta0,4)
		     *P1*( (1+2*log(L0))/pow(L0,2) - (1+2*log(L))/pow(L,2))
		     +pow(beta1,2)/27.0/pow(beta0,6)
		     *P1*((2.0+6*log(L0)+9*pow(log(L0),2))/pow(L0,3)
			  -(2.0+6*log(L)+9*pow(log(L),2))/pow(L,3) ) ) 

	       * pow( complex_d(finestruc1/finestruc0,0.0), -P0_qed/beta0_qed)

	       ); 
    }

  /**
   *
   * Returns the evolutionkernel
   * \f[
   *    E^z(Q^2,Q_0^2) 
   * \f]
   * for the singlet case with the inclusion of QED effects
   *
   * If the evolution goes over flavour threshold, the evolution is split
   * into separate pieces. For a fixed number of active flavour, the kernel
   * is obtained from evolutionkernel::singlet_with_qed_fixed_flavour.
   *
   */
  matrix_c evolutionkernel::singlet_with_qed(const complex_d & z, double Q, double Q0, int flag_qed)
    {
      //  mc < mtau < mb
      double mc   = alpha_strong::charm_threshold;
      double mtau = alpha_finestruc::tau_threshold;
      double mb   = alpha_strong::bottom_threshold;

      if ( Q0 < mc)
	{
	  if ( Q < mc )
	    { 
	      return evolutionkernel::singlet_with_qed_fixed_flavour(z,Q,Q0,2,1,2,flag_qed);
	    }
	  else if ( Q < mtau )
	    { 
	      return evolutionkernel::singlet_with_qed_fixed_flavour(z,Q,mc,2,2,2,flag_qed)
		* evolutionkernel::singlet_with_qed_fixed_flavour(z,mc,Q0,2,1,2,flag_qed);
	    }
	  else if ( Q < mb )
	    { 
	      return evolutionkernel::singlet_with_qed_fixed_flavour(z,Q,mtau,2,2,3,flag_qed)
		* evolutionkernel::singlet_with_qed_fixed_flavour(z,mtau,mc,2,2,2,flag_qed)
		* evolutionkernel::singlet_with_qed_fixed_flavour(z,mc,Q0,2,1,2,flag_qed);
	    }
	  else
	    { 
	      return evolutionkernel::singlet_with_qed_fixed_flavour(z,Q,mb,3,2,3,flag_qed)
		* evolutionkernel::singlet_with_qed_fixed_flavour(z,mb,mtau,2,2,3,flag_qed)
		* evolutionkernel::singlet_with_qed_fixed_flavour(z,mtau,mc,2,2,2,flag_qed)
		* evolutionkernel::singlet_with_qed_fixed_flavour(z,mc,Q0,2,1,2,flag_qed);
	    }
	}
      else if ( Q0 < mtau)
	{
	  if ( Q < mtau )
	    { 
	      return evolutionkernel::singlet_with_qed_fixed_flavour(z,Q,Q0,2,2,2,flag_qed);
	    }
	  else if ( Q < mb )
	    { 
	      return evolutionkernel::singlet_with_qed_fixed_flavour(z,Q,mtau,2,2,3,flag_qed)
		* evolutionkernel::singlet_with_qed_fixed_flavour(z,mtau,Q0,2,2,2,flag_qed);
	    }
	  else
	    { 
	      return evolutionkernel::singlet_with_qed_fixed_flavour(z,Q,mb,3,2,3,flag_qed)
		* evolutionkernel::singlet_with_qed_fixed_flavour(z,mb,mtau,2,2,3,flag_qed)
		* evolutionkernel::singlet_with_qed_fixed_flavour(z,mtau,Q0,2,2,2,flag_qed);
	    }
	}
      else if ( Q0 < mb)
	{
	  if ( Q < mb )
	    { 
	      return evolutionkernel::singlet_with_qed_fixed_flavour(z,Q,Q0,2,2,3,flag_qed);
	    }
	  else
	    { 
	      return evolutionkernel::singlet_with_qed_fixed_flavour(z,Q,mb,3,2,3,flag_qed)
		* evolutionkernel::singlet_with_qed_fixed_flavour(z,mb,Q0,2,2,3,flag_qed);
	    }
	}
      else
	{
	  return evolutionkernel::singlet_with_qed_fixed_flavour(z,Q,Q0,3,2,3,flag_qed);
	}

      // default, should not occur
      return matrix_c(0.0,5,5);

    }

  /**
   *
   */
  matrix_c  evolutionkernel::singlet_with_qed_fixed_flavour(const complex_d & z, double Q1, double Q0, 
					   int Nf_down, int Nf_up, int Nl, int flag_qed)
    {
      int Nf = Nf_down + Nf_up;

      // constants
      double beta0 = alpha_strong::beta_0(Nf);
      double beta1 = alpha_strong::beta_1(Nf);

      double beta0_qed = alpha_finestruc::beta_0(Nf_down, Nf_up, Nl);

      // up- and down-type quarks
      double delta_Nf = (Nf_up-Nf_down)/(Nf_up+Nf_down);

      double charge_plus  = 0.5*( pow(Q_up,2) + pow(Q_down,2) );
      double charge_minus = 0.5*( pow(Q_up,2) - pow(Q_down,2) );


      // get the anomalous dimensions
      complex_d gamma_0_qq = anomalous_dimensions::anomalous_0_qq(z,Nf);
      complex_d gamma_0_qg = anomalous_dimensions::anomalous_0_qg(z,Nf);
      complex_d gamma_0_gq = anomalous_dimensions::anomalous_0_gq(z,Nf);
      complex_d gamma_0_gg = anomalous_dimensions::anomalous_0_gg(z,Nf);

      complex_d gamma_0_NS = gamma_0_qq;
      complex_d gamma_1_NS = anomalous_dimensions::anomalous_1_NS(z,1,Nf);

      complex_d gamma_1_qq = anomalous_dimensions::anomalous_1_qq(z,Nf);
      complex_d gamma_1_qg = anomalous_dimensions::anomalous_1_qg(z,Nf);
      complex_d gamma_1_gq = anomalous_dimensions::anomalous_1_gq(z,Nf);
      complex_d gamma_1_gg = anomalous_dimensions::anomalous_1_gg(z,Nf);

      // divide out colour factors for qed anomalous dimensions
      complex_d gamma_0_ff = gamma_0_qq/CF;
      complex_d gamma_0_fp = gamma_0_qg/(2.0*TR*Nf);
      complex_d gamma_0_pf = gamma_0_gq/CF;
      complex_d gamma_0_pp = complex_d( 8.0/3.0, 0.0 );

      // set up kernel
      complex_d zero = complex_d(0.0,0.0);
      complex_d one  = complex_d(1.0,0.0);

      complex_d M0[25] = { gamma_0_NS,       zero, delta_Nf*gamma_0_qg, zero, zero,
			         zero, gamma_0_qq,          gamma_0_qg, zero, zero,
			         zero, gamma_0_gq,          gamma_0_gg, zero, zero,
			         zero,       zero,                zero, zero, zero,
			         zero,       zero,                zero, zero, zero  };

      matrix_c P0 = -0.5*matrix_c(M0,5,5);


      complex_d M1[25] = { gamma_1_NS, delta_Nf*(gamma_1_qq-gamma_1_NS), delta_Nf*gamma_1_qg, zero, zero,
			         zero,                       gamma_1_qq,          gamma_1_qg, zero, zero,
			         zero,                       gamma_1_gq,          gamma_1_gg, zero, zero,
			         zero,                             zero,                zero, zero, zero,
			         zero,                             zero,                zero, zero, zero  };

      matrix_c P1 = -0.5*matrix_c(M1,5,5);


      complex_d Mqed[25] = { 
	charge_plus*gamma_0_ff, charge_minus*gamma_0_ff, zero, 2.0*Nc*Nf*(charge_plus*delta_Nf+charge_minus)*gamma_0_fp, zero,
	charge_minus*gamma_0_ff, charge_plus*gamma_0_ff, zero, 2.0*Nc*Nf*(charge_plus+delta_Nf*charge_minus)*gamma_0_fp, zero,
	zero, zero, zero, zero, zero,
	charge_minus*gamma_0_pf, charge_plus*gamma_0_pf, zero, -3.0/4.0*beta0_qed*gamma_0_pp, gamma_0_pf,
	zero, zero, zero, 2.0*Nl*gamma_0_fp, gamma_0_ff };

      matrix_c Pqed = -0.5*matrix_c(Mqed,5,5);

      complex_d MUnit[25] = { one, zero, zero, zero, zero,
		             zero,  one, zero, zero, zero,
			     zero, zero,  one, zero, zero,
			     zero, zero, zero,  one, zero,
			     zero, zero, zero, zero,  one };

      matrix_c Unit = matrix_c(MUnit,5,5);

      // define L's
      double L1 = 2.0*log(Q1);  
      double L0 = 2.0*log(Q0); 

      matrix_c kernel = Unit;

      for (int iteration=1; iteration <= number_of_iterations; iteration++)
	{
	  double Lup  = L0 +  iteration   *(L1-L0)/number_of_iterations;
	  double Llow = L0 + (iteration-1)*(L1-L0)/number_of_iterations;

	  double Qup   = exp(Lup/2.0);
	  double Qlow = exp(Llow/2.0);

	  // alpha_s at the different scales
	  // calculate a_s/4/Pi
	  double strongup  = alpha_strong::get_value(Qup,Nf);
	  double stronglow = alpha_strong::get_value(Qlow,Nf);

	  double finestrucup  = alpha_finestruc::get_value(Qup,Nf_down,Nf_up,Nl);
	  double finestruclow = alpha_finestruc::get_value(Qlow,Nf_down,Nf_up,Nl);


	  kernel = ( Unit + ( strongup*P0 + pow(strongup,2)*P1 + flag_qed*finestrucup*Pqed )*(Lup-Llow) ) * kernel; 

	} // end iteration

      return( kernel );
    }

  // static data members

  int evolutionkernel::method = evolutionkernel::N_space;

  int evolutionkernel::number_of_iterations = 1000;

} // namespace pdf

