

/** @file integration_contour.cc
 *
 * Implementation of integration_contour 
 *
 */

/*
 *  Copyright (C) 2002 Stefan Weinzierl
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <cmath>
#include <stdexcept>

#include "constants_math.h"
#include "complex_d.h"
#include "matrix_c.h"
#include "matrix_d.h"
#include "special_functions.h"

#include "integration_contour.h"
#include "partondistribution.h"
#include "coupling.h"
#include "anomalous.h"
#include "evolutionkernel.h"

namespace pdf {

  /**
   *
   * This method returns
   * \f[
   *    \frac{c_2}{2 \pi} \mbox{Re}\; 
   *      \left( e^u \left( 1 - i c_2 c_3 \sqrt{u} \right) EF(x,z) \right)
   * \f]
   * where
   * \f[
   *   z = c_0 + i c_2 \sqrt{u} + \frac{1}{2} c_2^2 c_3 u,
   * \f] \f[
   *   c_2 = \sqrt{\frac{2 F F''}{F''^2 - F' F'''}},
   * \f] \f[
   *   c_3 = \frac{F'''}{3 F''},
   * \f] \f[
   *    F(x,z) = x^{-z} \sum\limits_{i=0}^{n-1}
   *                     A_i B(z+\alpha_i-1,1+\beta_i),
   * \f] \f[
   *    EF(x,z) = x^{-z} E^z(Q^2,Q_0^2) \sum\limits_{i=0}^{n-1}
   *                     A_i B(z+\alpha_i-1,1+\beta_i),
   * \f] 
   * and \f$c_0\f$ is the minimum of \f$F\f$ on the real axis right to the
   * rightmost pole.
   *
   */
  double integration_contour::nonsinglet(const partondistribution & f, double Q, double x, double u)
    {
      double c0=f.get_c0(x);

      // get the derivatives
      double F = f.get_F(x,c0);
      double dF = f.get_dF(x,c0);
      double ddF = f.get_ddF(x,c0);
      double dddF = f.get_dddF(x,c0);

      // define c2 and c3
      double c3 = dddF/3.0/ddF;
      double c2 = sqrt(2.0*F*ddF/(pow(ddF,2)-dF*dddF));

      complex_d z=complex_d(c0+c3/2.0*pow(c2,2)*u,c2*sqrt(u));

      complex_d evolution_operator;
      if ( f.with_qed() )
	{
	  evolution_operator = evolutionkernel::nonsinglet_with_qed(z,Q,f.get_Q0(),f.get_eta(),f.get_electric_charge());
	}
      else
	{
	  evolution_operator = evolutionkernel::nonsinglet(z,Q,f.get_Q0(),f.get_eta());
	}

      return ( 1.0/2.0/Pi*c2
	       *real( exp(u)*(1.0-complex_d(0.0,1.0)*c2*c3*sqrt(u))
		      * pow(complex_d(x,0.0),-z) 
		      * evolution_operator 
		      * f.get_Mellin_transform(z) 
		      ) 
	       );
    }

  /**
   *
   * Same as integration_contour::nonsinglet, but now for the singlet case.
   *
   */
  vector_d integration_contour::singlet(const partondistribution & s, const partondistribution & g, 
		   double Q, double x, double u)
    {
      // find z_0
      // Approximation : smallest alpha
      //               : smallest beta
      // here within sigmadistr
      double c0=(s+g).get_c0(x);
      complex_d z_0=complex_d(c0,0.0);

      // get the LO anomalous dimensions at z_0
      int Nf = alpha_strong::get_active_flavours( s.get_Q0() );
      complex_d gamma_0_qq_z0 = anomalous_dimensions::anomalous_0_qq(z_0,Nf);
      complex_d gamma_0_qg_z0 = anomalous_dimensions::anomalous_0_qg(z_0,Nf);
      complex_d gamma_0_gq_z0 = anomalous_dimensions::anomalous_0_gq(z_0,Nf);
      complex_d gamma_0_gg_z0 = anomalous_dimensions::anomalous_0_gg(z_0,Nf);

      // define eigenvalues of one-loop anomalous dimension matrix at z_0
      complex_d diskriminante_z0 = pow( pow( gamma_0_gg_z0-gamma_0_qq_z0 ,2) + 4.0*gamma_0_qg_z0*gamma_0_gq_z0 , 0.5);
      complex_d lambda_p_z0 = 0.5*(gamma_0_qq_z0+gamma_0_gg_z0+diskriminante_z0);
      complex_d lambda_m_z0 = 0.5*(gamma_0_qq_z0+gamma_0_gg_z0-diskriminante_z0);

      double csigma= real(-gamma_0_qg_z0);
      double cgluon= real(gamma_0_qq_z0-lambda_m_z0);

      // get the derivatives
      double F = csigma*s.get_F(x,c0) + cgluon*g.get_F(x,c0);
      double dF = csigma*s.get_dF(x,c0) + cgluon*g.get_dF(x,c0);
      double ddF = csigma*s.get_ddF(x,c0) + cgluon*g.get_ddF(x,c0);
      double dddF = csigma*s.get_dddF(x,c0) + cgluon*g.get_dddF(x,c0);

      // define c2 and c3
      double c3 = dddF/3.0/ddF;
      double c2 = sqrt(2.0*F*ddF/(pow(ddF,2)-dF*dddF));

      complex_d z=complex_d(c0+c3/2.0*pow(c2,2)*u,c2*sqrt(u));

      // Mellin transform of the structure functions
      complex_d Sigma = s.get_Mellin_transform(z);
      complex_d Gluon = g.get_Mellin_transform(z);

      complex_d vecstart[2] = { Sigma, Gluon };
      vector_c startdistr = vector_c(vecstart,2);

      vector_c res = 1.0/2.0/Pi*c2*exp(u)*(1.0-complex_d(0.0,1.0)*c2*c3*sqrt(u))*
	pow(complex_d(x,0.0),-z) * evolutionkernel::singlet(z,Q,s.get_Q0()) * startdistr;

      double res_real[2] = { real(res[0]), 
			     real(res[1]) };

      return vector_d(res_real,2);
    }

  /**
   *
   * Same as integration_contour::singlet, but now with the inclusion of QED effects.
   *
   */
  vector_d integration_contour::singlet_with_qed(const partondistribution & Delta, 
						 const partondistribution & s, const partondistribution & g, 
						 const partondistribution & phot, const partondistribution & lept, 
						 double Q, double x, double u)
    {
      partondistribution f = Delta + s + g + phot + lept;

      double c0=f.get_c0(x);

      // get the derivatives
      double F = f.get_F(x,c0);
      double dF = f.get_dF(x,c0);
      double ddF = f.get_ddF(x,c0);
      double dddF = f.get_dddF(x,c0);

      // define c2 and c3
      double c3 = dddF/3.0/ddF;
      double c2 = sqrt(2.0*F*ddF/(pow(ddF,2)-dF*dddF));

      complex_d z=complex_d(c0+c3/2.0*pow(c2,2)*u,c2*sqrt(u));

      // Mellin transform of the structure functions
      complex_d Delta_UD = Delta.get_Mellin_transform(z);
      complex_d Sigma    = s.get_Mellin_transform(z);
      complex_d Gluon    = g.get_Mellin_transform(z);
      complex_d Photon   = phot.get_Mellin_transform(z);
      complex_d Lepton   = lept.get_Mellin_transform(z);

      complex_d vecstart[5] = { Delta_UD, Sigma, Gluon, Photon, Lepton };
      vector_c startdistr = vector_c(vecstart,5);

      int flag_qed;;
      if ( Delta.with_qed() )
	{
	  flag_qed = 1;
	}
      else
	{
	  flag_qed = 0;
	}

      vector_c res = 1.0/2.0/Pi*c2*exp(u)*(1.0-complex_d(0.0,1.0)*c2*c3*sqrt(u))*
	pow(complex_d(x,0.0),-z) * evolutionkernel::singlet_with_qed(z,Q,Delta.get_Q0(),flag_qed) * startdistr;

      double res_real[5] = { real(res[0]),
			     real(res[1]), 
			     real(res[2]), 
			     real(res[3]), 
			     real(res[4]) };

      return vector_d(res_real,5);
    }

} // namespace pdf

