

/** @file partondistribution.cc
 *
 * Implementation of partondistribution
 *
 */

/*
 *  Copyright (C) 2002 Stefan Weinzierl
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <valarray>
#include <stdexcept>

#include "complex_d.h"
#include "special_functions.h"

#include "partondistribution.h"

namespace pdf {

  /**
   *
   * Default constructor
   *
   */
  partondistribution::partondistribution(void)
    {} 

  /**
   *
   * Standard constructor
   *
   */
  partondistribution::partondistribution(int nr, double Q0r, 
					 double Ar[], double alphar[], double betar[], int etar) : 
    n(nr), Q0(Q0r), A(Ar,nr), alpha(alphar,nr), beta(betar,nr), eta(etar)
    {}

  /**
   *
   * Standard constructor from valarray
   *
   */
  partondistribution::partondistribution(int nr, double Q0r, 
					 const std::valarray<double> & Ar, 
					 const std::valarray<double> & alphar, 
					 const std::valarray<double> & betar, int etar ) :
    n(nr), Q0(Q0r), A(Ar), alpha(alphar), beta(betar), eta(etar)
  { }

  /**
   *
   * Returns \f$Q_0\f$.
   *
   */
  double partondistribution::get_Q0(void) const
    {
      return Q0;
    }

  /**
   *
   * Returns \f$\eta\f$.
   *
   */
  int partondistribution::get_eta(void) const
    {
      return eta;
    }

  /**
   *
   * Evaluates the Mellin transform of the parton distribution 
   * \f[
   *    f^z(Q_0^2) = \sum\limits_{i=0}^{n-1}
   *                     A_i B(z+\alpha_i-1,1+\beta_i)
   * \f]
   * at the point \f$z\f$.
   *
   */
  complex_d partondistribution::get_Mellin_transform(complex_d z) const
    {
      complex_d erg = 0.0;

      for(int i=0;i<n;i++)
	{
	  erg = erg + A[i]*Beta(z+alpha[i]-1.0,complex_d(beta[i]+1,0.0));
	}

      return(erg);
    }

  /**
   *
   * Evaluates the function
   * \f[
   *    F(x,z) = x^{-z} \sum\limits_{i=0}^{n-1}
   *                     A_i B(z+\alpha_i-1,1+\beta_i).
   * \f]
   * for the given pair \f$(x,z)\f$.
   *
   */
  double partondistribution::get_F(double x, double z) const
    {
      int i;

      double F;

      double sum_F = 0.0;

      for(i=0;i<n;i++)
	{
	  F = pow(x,-z)*A[i]*Beta(z+alpha[i]-1.0,1.0+beta[i]);

	  sum_F = sum_F + F;
	}

      return(sum_F);
    }

  /**
   *
   * Evaluates 
   * \f[
   *    \frac{d}{dz} F(x,z) = \sum\limits_{i=0}^{n-1} G_i' F_i
   * \f]
   * where
   * \f[
   *         F_i = x^{-z} A_i B(z+\alpha_i-1,1+\beta_i),
   * \f] \f[
   *         G_i = \ln \left( x^{-z} B(z+\alpha_i-1,1+\beta_i) \right)
   * \f] \f[
   *         G_i' = -\ln x + \Psi(z+\alpha_i-1) - \Psi(z+\alpha_i+\beta_i)
   * \f] 
   *
   */
  double partondistribution::get_dF(double x, double z) const
    {
      int i;
      double dG;

      double F;
      double dF;

      double sum_dF = 0.0;

      for(i=0;i<n;i++)
	{
	  dG   =-log(x)+Psi(z+alpha[i]-1.0) -Psi(z+alpha[i]+beta[i]);

	  F    = pow(x,-z)*A[i]*Beta(z+alpha[i]-1.0,1.0+beta[i]);
	  dF   =   dG * F;

	  sum_dF = sum_dF + dF;
	}

      return(sum_dF);
    }

  /**
   *
   * Evaluates 
   * \f[
   *    \frac{d^2}{dz^2} F(x,z) = \sum\limits_{i=0}^{n-1} \left( G_i'' + G_i'^2 \right) F_i
   * \f]
   * where
   * \f[
   *         G_i'' = \Psi'(z+\alpha_i-1) - \Psi'(z+\alpha_i+\beta_i)
   * \f] 
   *
   */
  double partondistribution::get_ddF(double x, double z) const
    {
      int i;
      double dG;
      double ddG;

      double F;
      double ddF;

      double sum_ddF = 0.0;

      for(i=0;i<n;i++)
	{
	  dG   =-log(x)+Psi(z+alpha[i]-1.0) -Psi(z+alpha[i]+beta[i]);
	  ddG  =        Psi1(z+alpha[i]-1.0)-Psi1(z+alpha[i]+beta[i]);

	  F    = pow(x,-z)*A[i]*Beta(z+alpha[i]-1.0,1.0+beta[i]);
	  ddF  = ( ddG + pow(dG,2) ) * F;

	  sum_ddF = sum_ddF + ddF;
	}

      return(sum_ddF);
    }

  /**
   *
   * Evaluates 
   * \f[
   *    \frac{d^3}{dz^3} F(x,z) = \sum\limits_{i=0}^{n-1} \left( G_i''' + 3 G_i'' G_i' + G_i'^3 \right) F_i
   * \f]
   * where
   * \f[
   *         G_i''' = \Psi''(z+\alpha_i-1) - \Psi''(z+\alpha_i+\beta_i)
   * \f] 
   *
   */
  double partondistribution::get_dddF(double x, double z) const
    {
      int i;
      double dG;
      double ddG;
      double dddG;

      double F;
      double dddF;

      double sum_dddF = 0.0;

      for(i=0;i<n;i++)
	{
	  dG   =-log(x)+Psi(z+alpha[i]-1.0) -Psi(z+alpha[i]+beta[i]);
	  ddG  =        Psi1(z+alpha[i]-1.0)-Psi1(z+alpha[i]+beta[i]);
	  dddG =        Psi2(z+alpha[i]-1.0)-Psi2(z+alpha[i]+beta[i]);

	  F    = pow(x,-z)*A[i]*Beta(z+alpha[i]-1.0,1.0+beta[i]);
	  dddF = ( dddG + 3.0*ddG*dG + pow(dG,3) ) * F;

	  sum_dddF = sum_dddF + dddF;
	}

      return(sum_dddF);
    }

  /**
   *
   * Returns for fixed \f$x\f$ the position of the minimum in the \f$z\f$-plane
   * of the function
   * \f[
   *    F(x,z) = x^{-z} \sum\limits_{i=0}^{n-1}
   *                     A_i B(z+\alpha_i-1,1+\beta_i).
   * \f]
   * on the real axis right to the rightmost pole.
   *
   */
  double partondistribution::get_c0(double x) const
    {
      int i;
      double a,b;

      // find minimal value of alpha and maximal of beta
      a=alpha[0];
      b=beta[0];

      for(i=0;i<n;i++)
	{
	  if ( alpha[i] < a )
	    a = alpha[i];
	  if (beta[i] > b )
	    b = beta[i];
	}

      // return Newton_Raphson

      return( root_safeI(a,b,x) );
    }

  /**
   *
   * Solves numerically the equation
   * \f[
   *    \frac{d}{dz} F(x,z) = 0 
   * \f]
   * for \f$z\f$.
   *
   */
  double partondistribution::root_safeI(double a, double b, double x) const
    {
      double f,df,flow,fhigh;
      double x1,x2,xlow,xhigh,rts,dxold,dx;
      int j;
      double temp;

      // initial guess and brackets
      x1=1-a+0.0001;
      x2=1-a+0.0001-(1+b)/log(x); 

      flow = get_dF(x,x1);
      fhigh= get_dF(x,x2);

      if (flow*fhigh >= 0.0)
	{ 
	  throw(std::runtime_error("Root must be bracketed "));
	}
      if (flow < 0.0)  // orient search so that f(x1)<0
	{ 
	  xlow = x1;
	  xhigh= x2;
	}
      else
	{
	  xhigh = x1;
	  xlow  = x2;
	}
      rts=1-a-1.0/log(x); // initial guess for the root
      dxold=fabs(x2-x1);
      dx=dxold;

      f = get_dF(x,rts);
      df = get_ddF(x,rts);

      for (j=1;j<=100;j++)
	{
	  if ( (((rts-xhigh)*df-f)*((rts-xlow)*df-f) >= 0.0) // bisect if Newton out of range
	       || (fabs(2.0*f) > fabs(dxold*df)) )            // or not decreasing fast enough
	    {
	      dxold=dx;
	      dx=0.5*(xhigh-xlow);
	      rts=xlow+dx;
	      if (xlow == rts) return rts;    //  change in root negligible
	    }
	  else
	    {
	      dxold=dx;
	      dx=f/df;
	      temp=rts;
	      rts -= dx;
	      if (temp == rts) return rts;   //  change in root negligible
	    }
	  if (fabs(dx) < 1E-8) return rts;   // convergence criterion
	  f = get_dF(x,rts);
	  df= get_ddF(x,rts);
	  if (f < 0.0 ) 
	    xlow = rts;
	  else
	    xhigh = rts;
	}

      throw(std::runtime_error("Maximum number of iterations exceeded in root_safe "));

    }

  // friends

  /**
   *
   * Addition of two parton distributions
   *
   * The sum of two parton distributions is given by
   * \f[
   *    \sum\limits_{i=0}^{n-1}
   *                     A_i x^{\alpha_i} (1-x)^{\beta_i}
   *    + \sum\limits_{i=0}^{n'-1}
   *                     A_i' x^{\alpha_i'} (1-x)^{\beta_i'}
   * \f]
   *
   * The values of \f$Q_0\f$ and \f$\eta\f$ of the two summands have to agree,
   * otherwise the function throws an exception.
   *
   */
  partondistribution operator+ (const partondistribution & f1, const partondistribution & f2)
  {
    if ( f1.Q0 != f2.Q0 ) throw(std::logic_error("the value of Q0 must be equal to add two pdf's"));
    if ( f1.eta != f2.eta ) throw(std::logic_error("eta must be equal to add two pdf's"));

    std::valarray<double> new_A(f1.n+f2.n);
    std::valarray<double> new_alpha(f1.n+f2.n);
    std::valarray<double> new_beta(f1.n+f2.n);

    for( int i=0; i<f1.n; i++)
      {
	new_A[i] = f1.A[i];
	new_alpha[i] = f1.alpha[i];
	new_beta[i] = f1.beta[i];
      }
    for( int i=0; i<f2.n; i++)
      {
	new_A[f1.n+i] = f2.A[i];
	new_alpha[f1.n+i] = f2.alpha[i];
	new_beta[f1.n+i] = f2.beta[i];
      }

    return partondistribution(f1.n+f2.n,f1.Q0,new_A,new_alpha,new_beta,f1.eta);
  }


  /**
   *
   * Default constructor
   *
   */
  partondistribution_with_qed::partondistribution_with_qed(void) : partondistribution() 
    {} 

  /**
   *
   * Standard constructor
   *
   */
  partondistribution_with_qed::partondistribution_with_qed(int nr, double Q0r, 
					 double Ar[], double alphar[], double betar[], double electric_charger, int etar) : 
    partondistribution(nr,Q0r,Ar,alphar,betar,etar), electric_charge(electric_charger)
    {}

  /**
   *
   * Standard constructor from valarray
   *
   */
  partondistribution_with_qed::partondistribution_with_qed(int nr, double Q0r, 
					 const std::valarray<double> & Ar, 
					 const std::valarray<double> & alphar, 
					 const std::valarray<double> & betar, double electric_charger, int etar ) :
    partondistribution(nr,Q0r,Ar,alphar,betar,etar), electric_charge(electric_charger)
  { }

  // friends

  /**
   *
   * Addition of two parton distributions
   *
   * The values of \f$Q_0\f$, the electric charge and \f$\eta\f$ of the two summands have to agree,
   * otherwise the function throws an exception.
   *
   */
  partondistribution_with_qed operator+ (const partondistribution_with_qed & f1, const partondistribution_with_qed & f2)
  {
    if ( f1.Q0 != f2.Q0 ) throw(std::logic_error("the value of Q0 must be equal to add two pdf's"));
    if ( f1.eta != f2.eta ) throw(std::logic_error("eta must be equal to add two pdf's"));
    if ( f1.electric_charge != f2.electric_charge ) throw(std::logic_error("electric charge must be equal to add two pdf's"));

    std::valarray<double> new_A(f1.n+f2.n);
    std::valarray<double> new_alpha(f1.n+f2.n);
    std::valarray<double> new_beta(f1.n+f2.n);

    for( int i=0; i<f1.n; i++)
      {
	new_A[i] = f1.A[i];
	new_alpha[i] = f1.alpha[i];
	new_beta[i] = f1.beta[i];
      }
    for( int i=0; i<f2.n; i++)
      {
	new_A[f1.n+i] = f2.A[i];
	new_alpha[f1.n+i] = f2.alpha[i];
	new_beta[f1.n+i] = f2.beta[i];
      }

    return partondistribution_with_qed(f1.n+f2.n,f1.Q0,new_A,new_alpha,new_beta,f1.electric_charge,f1.eta);
  }


} // namespace pdf

