
/** @file partondistribution.h
 *
 * Interface to partondistribution
 *
 */

/*
 *  Copyright (C) 2002 Stefan Weinzierl
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __PDF_PARTONDISTRIBUTION_H__
#define __PDF_PARTONDISTRIBUTION_H__

#include <valarray>

#include "complex_d.h"
 
namespace pdf {
 
  /**
   *
   * The class partondistribution represents the parameterization
   * of a parton distribution in the form
   * \f[
   *    x f(x,Q_0^2) = \sum\limits_{i=0}^{n-1}
   *                     A_i x^{\alpha_i} (1-x)^{\beta_i}
   * \f]
   *
   * In addition this class has a data member
   * \f$\eta = \pm  1\f$, which selects odd or even
   * moments for the evolution.
   * The default value for \f$\eta\f$ in the constructor is \f$+1\f$,
   * which is good for the singlet distributions and non-singlet
   * non-valence distributions like
   * \f[
   *    (u+\bar{u}) - ( d + \bar{d} ),
   * \f]
   * \f[
   *    (u+\bar{u}) + ( d + \bar{d} ) - 2 ( s + \bar{s} ), ...
   * \f]
   * Valence non-singlet distributions like
   * \f[
   *     (u-\bar{u}),
   *     (d - \bar{d}), ...
   * \f]
   * need \f$\eta=-1\f$, which has to passed explicitly to the constructor.
   *
   */
  class partondistribution {
    // ctors
    public :  
      partondistribution(void);
      partondistribution(int n, double Q0, 
			 double A[], double alpha[], double beta[], int eta = 1);
      partondistribution(int n, double Q0, 
			 const std::valarray<double> & A, 
			 const std::valarray<double> & alpha, const std::valarray<double> & beta, int eta = 1);

    // member functions
    public :
      double get_Q0(void) const;
      int get_eta(void) const;

      complex_d get_Mellin_transform(complex_d z) const;
      double get_F(double x, double z) const;
      double get_dF(double x, double z) const;
      double get_ddF(double x, double z) const;
      double get_dddF(double x, double z) const;
      double get_c0(double x) const;

    private :
      double root_safeI(double a, double b, double x) const;

    // virtual member functions
    public :

      /**
       *
       * Return false for a partondistribution.
       * The method is overridden by the derived class
       * partondistribution_with_qed. 
       *
       */
      virtual bool with_qed(void) const {return false;}

      /**
       *
       * Return zero for a partondistribution.
       * The method is overridden by the derived class
       * partondistribution_with_qed. 
       *
       */
      virtual double get_electric_charge(void) const {return 0.0;}

    // friends
      friend partondistribution operator+ (const partondistribution & f1, const partondistribution & f2);

    // data members
    protected : 
      int     n;
      double Q0;
      std::valarray<double> A;
      std::valarray<double> alpha;
      std::valarray<double> beta;
      int eta;
  };


  /**
   *
   * The class partondistribution_with_qed.
   *
   */
  class partondistribution_with_qed : public partondistribution {
    // ctors
    public :  
      partondistribution_with_qed(void);
      partondistribution_with_qed(int n, double Q0, 
				  double A[], double alpha[], double beta[], double electric_charge = 0.0, int eta = 1);
      partondistribution_with_qed(int n, double Q0, 
				  const std::valarray<double> & A, 
				  const std::valarray<double> & alpha, const std::valarray<double> & beta, 
				  double electric_charge, int eta = 1);

    // overriding virtual member functions from base class
    public :

      /**
       *
       * Return true for a partondistribution_with_qed.
       *
       */
      bool with_qed(void) const {return true;}

      /**
       *
       * Returns the electric charge of the parton.
       *
       */
      double get_electric_charge(void) const {return electric_charge;}

    // friends
      friend partondistribution_with_qed operator+ (const partondistribution_with_qed & f1, const partondistribution_with_qed & f2);

    // data members
    protected : 
      /// the electric charge of the parton
      double electric_charge;

  };


} // namespace pdf

#endif // ndef __PDF_PARTONDISTRIBUTION_H__

