

/** @file special_functions.cc
 *
 * Implementation of special functions
 *
 */

/*
 *  Copyright (C) 2002 Stefan Weinzierl
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <cmath>
#include <stdexcept>

#include "special_functions.h"
#include "constants_math.h"

//#define STILDE_WITH_M1

namespace pdf {

  /**
   *
   * Cotangens
   *
   */
  complex_d cot(const complex_d & xx)
  {
    if ( imag(xx) > 60.0 )
      {
	return( complex_d(0.0,-1.0) );
      }
    else if ( imag(xx) < -60.0 ) 
      {
	return( complex_d(0.0,1.0) );
      }

    // a compiler bug in gcc 2.95.3 prevents us from writing
    //   1.0/tan(xx)
    // this bug does not appear in gcc 3.1
    return( cos(xx)/sin(xx) );
  }

  /**
   *
   * The log of the sin function.
   *
   */
  complex_d LogSin(const complex_d & xx)
  {
    if ( imag(xx) > 60.0 )
      {
	return -complex_d(0.0,1.0)*xx - log( complex_d(0.0, -2.0));
      }
    else if ( imag(xx) < -60.0 ) 
      {
	return complex_d(0.0,1.0)*xx - log( complex_d(0.0, 2.0));
      }

    return log(sin(xx));
  }

  /**
   *
   * The Gamma function \f$ \Gamma(x) \f$ with complex argument.
   *
   * If \f$ \mbox{Re}(x) < 1 \f$ we use the reflection identity
   * \f[
   *    \Gamma(x) = \frac{\pi (1-x)}{\sin \left( \pi (1-x) \right)} \frac{1}{\Gamma(2-x)}
   * \f]
   *
   * For \f$ \mbox{Re}(x) > 1 \f$ we use Lanczos's formula:
   * \f[
   *     \Gamma(x) = \left( x + \gamma_E - \frac{1}{2} \right)^{z-\frac{1}{2}} 
   *                   e^{- \left(x + \gamma_E - \frac{1}{2} \right)}
   *                 \sqrt{2 \pi }
   *                 \left[
   *                         c_0 + \frac{c_1}{x} + \frac{c_2}{x+1} + \frac{c_3}{x+3} + ...
   *                 \right]
   * \f]
   *
   */
  complex_d Gamma(const complex_d & xx)
  {
    complex_d x,tmp,ser;
    static double cof[6]={76.18009173,-86.50532033,24.01409822,
			  -1.231739516,0.12085800e-2,-0.536382e-5};

    if (real(xx) < 1.0 ) // -------------------------------
      {
	return (  Pi*(1.0-xx)/sin(Pi*(1.0-xx))/Gamma(2.0-xx) );
      }
    else   // case   real(xx) > 1.0 ------------------------
      {
	x=xx-1.0;
	tmp=x+5.5;
	tmp = pow(tmp,(x+0.5))*exp(-tmp);
	ser=complex_d(1.0,0.0);
	for (int j=0;j<=5;j++) 
	  {
	    x = x+1.0;
	    ser = ser+cof[j]/x;
	  }
	return tmp*2.50662827465*ser;
      }
  }

  /**
   *
   * The log of the Gamma function \f$ \ln \Gamma(x) \f$.
   *
   * Since the Gamma functions grows quite fast, it's often
   * better to calculate \f$ \ln \Gamma(x) \f$ in order to avoid
   * overflow errors.
   *
   */
  complex_d LogGamma(const complex_d & xx)
  {
    complex_d x,tmp,ser;
    static double cof[6]={76.18009173,-86.50532033,24.01409822,
			  -1.231739516,0.12085800e-2,-0.536382e-5};

    if (real(xx) < 1.0 ) // -------------------------------
      {
	return (  log(Pi*(1.0-xx)) - LogSin(Pi*(1.0-xx)) - LogGamma(2.0-xx) );
      }
    else   // case   real(xx) > 1.0 ------------------------
      {
	x=xx-1.0;
	tmp=x+5.5;
	tmp = tmp-(x+0.5)*log(tmp);
	ser=complex_d(1.0,0.0);
	for (int j=0;j<=5;j++) 
	  {
	    x = x+1.0;
	    ser = ser+cof[j]/x;
	  }
	return -tmp+log(2.50662827465*ser);
      }
  }

  /**
   *
   * The incomplete Gamma function
   * \f[
   *    \Gamma(a,x) = \int\limits_x^\infty e^{-t} t^{a-1} dt, \;\;\; a>0
   * \f]
   *
   */
  complex_d IncompleteGamma(const complex_d & a, double x)
  {
    int n;
    
    if ( x < (real(a)+1.0) )
      { // use series approximation ------------------------------
	complex_d sum,del,ap;
	ap=a;
	del=sum=1.0/a;
	for(n=1; n<=100; n++)
	  {
	    ap = ap+1.0;
	    del = del*x/ap;
	    sum = sum+del;
	    if ( norm(del) < norm(sum)*1E-14)
	      {
		return( Gamma(a)*(1.0-sum*exp(-x+a*log(x)-LogGamma(a))) );
	      }
	  }	
	throw(std::runtime_error("Error in IncompleteGamma (series) "));
      }
    else
      { // use continued fraction approximation ----------------------
	complex_d gold=complex_d(0.0,0.0);
	complex_d g;
	complex_d fac=complex_d(1.0,0.0);
	complex_d anf,ana,an;

	complex_d a0=complex_d(1.0,0.0);
	complex_d a1=complex_d(x,0.0);
	complex_d b0=complex_d(0.0,0.0);
	complex_d b1=complex_d(1.0,0.0);

	for (n=1; n<=100;n++)
	  {
	    an=complex_d(n,0.0);
	    ana=an-a;
	    a0=(a1+a0*ana)*fac;
	    b0=(b1+b0*ana)*fac;
	    anf=an*fac;
	    a1=x*a0+anf*a1;
	    b1=x*b0+anf*b1;
	    if ( norm(a1) )
	      {
		fac=1.0/a1;
		g=b1*fac;
		if (norm((g-gold)/g) < 1E-14)
		  { return ( Gamma(a)*exp(-x+a*log(x)-LogGamma(a))*g);
		  }
		gold=g;
	      }
	  }
	throw(std::runtime_error("Error in IncompleteGamma (fraction) "));
      }
  }

  /**
   *
   * The beta function
   * \f[
   *     B(x,y) = \frac{\Gamma(x) \Gamma(y)}{\Gamma(x+y)}
   * \f]
   *
   */
  complex_d Beta(const complex_d & x, const complex_d & y)
  {
    return ( exp(LogGamma(x)+LogGamma(y)-LogGamma(x+y)) );
  }

  /**
   *
   * The hypergeometric function
   * \f[
   *    {}_2F_1(a,b;c;z) =
   *       \sum\limits_{n=0}^\infty \frac{(a)_n (b)_n}{(c)_n} \frac{z^n}{n!}
   * \f]
   *
   * This routine approximates the hypergeometric function by the first ten terms
   * of the series expansion.
   *
   */
  complex_d HyperGeometric(const complex_d & a, const complex_d & b, const complex_d & c, const complex_d & z)
  {
    int n;
    complex_d la = LogGamma(a);
    complex_d lb = LogGamma(b);
    complex_d lc = LogGamma(c);

    complex_d erg = complex_d(0.0,0.0);

    // calculate first 10 terms
    for (n=0;n<10;n++)
      {
	erg=erg+exp(LogGamma(a+double(n))+LogGamma(b+double(n))-LogGamma(c+double(n))-la-lb+lc-LogGamma(double(n)+1.0))*pow(z,n);
      }
    return( erg );

  }

  /**
   *
   * \f[ 
   *     \Psi(z) = \frac{d}{dz} \ln \Gamma(z) = \frac{1}{\Gamma(z)} \frac{d}{dz} \Gamma(z)
   * \f]
   *
   */
  complex_d Psi(const complex_d & z)
  {

    if (real(z) < 0.0)
      {
	return( Psi(1.0-z) - Pi*cot(Pi*z) );
      }
    else if (real(z) < 10.0)
      {
	return( -1.0/z+Psi(z+1.0) );
      }
    else     // case real z > 10
      {
	return( log(z)-0.5/z-1./12./pow(z,2)+1./120./pow(z,4)-1./252./pow(z,6) );
      }
  }

  /**
   *
   * \f[ 
   *     \Psi(1,z) = \Psi'(z) = \frac{d}{dz} \Psi(z) = \frac{d^2}{dz^2} \ln \Gamma(z)
   * \f]
   *
   */
  complex_d Psi1(const complex_d & z)
  {

    if (real(z) < 0.0)
      {
	return( -Psi1(1.0-z) + pow(Pi,2)*(1.0 + pow(cot(Pi*z),2)) );
      }
    else if (real(z) < 10.0)
      {
	return( 1.0/pow(z,2)+Psi1(z+1.0) );
      }
    else     // case real z > 10
      {
	return( 1./z+1./2./pow(z,2)+1./6./pow(z,3)-1./30./pow(z,5)+1./42./pow(z,7)-1./30./pow(z,9) );
      }
  }

  /**
   *
   * \f[ 
   *     \Psi(2,z) = \Psi''(z) = \frac{d}{dz} \Psi(1,z) = \frac{d^3}{dz^3} \ln \Gamma(z)
   * \f]
   *
   */
  complex_d Psi2(const complex_d & z)
  {

    if (real(z) < 0.0)
      {
	return( Psi2(1.0-z) - 2.0*pow(Pi,3)*cot(Pi*z)*(1.0 + pow(cot(Pi*z),2)) );
      }
    else if (real(z) < 10.0)
      {
	return( -2.0/pow(z,3)+Psi2(z+1.0) );
      }
    else     // case real z > 10
      {
	return( -1.0/pow(z,2)-1.0/pow(z,3)-1.0/2.0/pow(z,4)+1.0/6.0/pow(z,6)-1.0/6.0/pow(z,8)+3.0/10.0/pow(z,10)-5.0/6.0/pow(z,12) );
      }
  }

  /**
   *
   * \f[ 
   *     \Psi(3,z) = \Psi'''(z) = \frac{d}{dz} \Psi(2,z) = \frac{d^4}{dz^4} \ln \Gamma(z)
   * \f]
   *
   */
  complex_d Psi3(const complex_d & z)
  {

    if (real(z) < 0.0)
      {
	return( -Psi3(1.0-z) + 2.0*pow(Pi,4)*(1.0+3.0*pow(cot(Pi*z),2))*(1.0+pow(cot(Pi*z),2)) );
      }
    else if (real(z) < 10.0)
      {
	return( +6.0/pow(z,4)+Psi3(z+1.0) );
      }
    else     // case real z > 10
      {
	return( 2.0/pow(z,3)+3.0/pow(z,4)+2.0/pow(z,5)-1.0/pow(z,7)+4.0/3.0/pow(z,9)-3.0/pow(z,11)+10.0/pow(z,13) );
      }
  }

  /**
   *
   * Real version of the cotangent function.
   *
   */
  double cot(double xx)
  { 
    return( 1.0/tan(xx) );
  }

  /**
   *
   * Real version of the log of the sin function.
   *
   */
  double LogSin(double xx)
  { 
    return log(sin(xx));
  }

  /**
   *
   * Real version of the Gamma function.
   *
   */
  double Gamma(double xx)
  { 
    return real(Gamma(complex_d(xx,0.0)));
  }

  /**
   *
   * Real version of the LogGamma function.
   *
   */
  double LogGamma(double xx)
  { 
    return real(LogGamma(complex_d(xx,0.0)));
  }

  /**
   *
   * Real version of the Beta function.
   *
   */
  double Beta(double x, double y)
  { 
    return real(Beta(complex_d(x,0.0),complex_d(y,0.0)));
  }

  /**
   *
   * Real version of the function \f$\Psi(z)\f$.
   *
   */
  double Psi(double z)
  {
    return real(Psi(complex_d(z,0.0)));
  }

  /**
   *
   * Real version of the function \f$\Psi(1,z)\f$.
   *
   */
  double Psi1(double z)
  {
    return real(Psi1(complex_d(z,0.0)));
  }

  /**
   *
   * Real version of the function \f$\Psi(2,z)\f$.
   *
   */
  double Psi2(double z)
  {
    return real(Psi2(complex_d(z,0.0)));
  }

  /**
   *
   * Real version of the function \f$\Psi(3,z)\f$.
   *
   */
  double Psi3(double z)
  {
    return real(Psi3(complex_d(z,0.0)));
  }

  /**
   *
   * Complex continuation of 
   * \f[
   *     S_1(N) = \sum\limits_{j=1}^N \frac{1}{j} 
   *            = \gamma_E + \Psi(N+1)
   * \f]
   *
   */
  complex_d S1(const complex_d & z)
  {
    return ( Euler+Psi(z+1.0) );
  }

  /**
   *
   * Complex continuation of 
   * \f[
   *     S_2(N) = \sum\limits_{j=1}^N \frac{1}{j^2} 
   *            = \zeta_2 - \Psi'(N+1)
   * \f]
   *
   */
  complex_d S2(const complex_d & z)
  {
    return ( zeta2-Psi1(z+1.0) );
  }

  /**
   *
   * Complex continuation of 
   * \f[
   *     S_3(N) = \sum\limits_{j=1}^N \frac{1}{j^3} 
   *            = \zeta_3 + \frac{1}{2} \Psi''(N+1)
   * \f]
   *
   */
  complex_d S3(const complex_d & z)
  {
    return( zeta3+1.0/2.0*Psi2(z+1.0) );
  }

  /**
   *
   * Complex continuation of 
   * \f[
   *     \tilde{S}(N) = \sum\limits_{j=1}^N \frac{(-1)^j}{j^2} S_1(j) 
   * \f]
   * with \f$ \eta = (-1)^N \f$ selecting the odd or even moments.
   *
   */
  complex_d Stilde(const complex_d & z, int eta)
  {

#ifdef STILDE_WITH_M1

    // 9.3.2004 use Kosowers method
    return( -5.0/8.0*zeta3+double(eta)*(S1(z)/pow(z,2)-zeta2/2.0*(Psi(z/2.0+0.5)-Psi(z/2.0))
					+ M1_helper(z+1.0) ));

#else // STILDE_WITH_M1

    // if we don't shift the argument :
    //   can use approximation of Gonzalez-Arroyo for z > 5
    //   and GRV for z < 5
    //
    // GRV : typo corrected

    if( norm(z) > 24.0 )
      {
	// Approximation Gonzalez-Arroyo, Lopez and Yndurain
	return( -5.0/8.0*zeta3+eta/2.0*( log(z)*(z-1.0)/pow(z,3) + Euler/pow(z,2) + (1.0-Euler)/pow(z,3)
				      -5.0/6.0/pow(z,4) ) );
      }
    else 
      {
	// GRV approximation
	return( -5.0/8.0*zeta3+double(eta)*(S1(z)/pow(z,2)-zeta2/2.0*(Psi(z/2.0+0.5)-Psi(z/2.0))
				 +(1.010/(z+1.0)-0.846/(z+2.0)+1.155/(z+3.0)-1.074/(z+4.0)+0.550/(z+5.0) ) ) );
      }

#endif // not STILDE_WITH_M1

  }

  /**
   *
   * Complex continuation of 
   * \f[
   *     S_1'(N/2) = \frac{1}{2} \left( 1 + (-1)^N \right) S_1(N/2)
   *               + \frac{1}{2} \left( 1 - (-1)^N \right) S_1((N-1)/2) 
   * \f]
   * with \f$ \eta = (-1)^N \f$ selecting the odd or even moments.
   *
   */
  complex_d Sprim1(const complex_d & z, int eta)  
  {
    return( (1+eta)/2.0*S1(z)+(1-eta)/2.0*S1(z-0.5) );
  }

  /**
   *
   * Complex continuation of 
   * \f[
   *     S_2'(N/2) = \frac{1}{2} \left( 1 + (-1)^N \right) S_2(N/2)
   *               + \frac{1}{2} \left( 1 - (-1)^N \right) S_2((N-1)/2) 
   * \f]
   * with \f$ \eta = (-1)^N \f$ selecting the odd or even moments.
   *
   */
  complex_d Sprim2(const complex_d & z, int eta)
  {
    return( (1+eta)/2.0*S2(z)+(1-eta)/2.0*S2(z-0.5) );
  }

  /**
   *
   * Complex continuation of 
   * \f[
   *     S_3'(N/2) = \frac{1}{2} \left( 1 + (-1)^N \right) S_3(N/2)
   *               + \frac{1}{2} \left( 1 - (-1)^N \right) S_3((N-1)/2) 
   * \f]
   * with \f$ \eta = (-1)^N \f$ selecting the odd or even moments.
   *
   */
  complex_d Sprim3(const complex_d & z, int eta)
  {
    return( (1+eta)/2.0*S3(z)+(1-eta)/2.0*S3(z-0.5) );
  }

  /**
   *
   * The function is defined by
   * \f[
   *    M_1(z) = \int\limits_0^1 dx x^{z-2} \frac{\mbox{Li}_2(x)}{x+1}
   * \f]
   * For \f$ \mbox{Re} z \f$ large, one can use the expansion
   * \f[
   *    M_1(z) = \frac{1}{2} \sum\limits_{j=0}^N \left( \frac{1}{2} \right)^j
   *               \sum\limits_{l=0}^j \left( \begin{array}{c} j \\ l \\ \end{array} \right)
   *                                   \left( -1 \right)^l M_2(z+l)
   *             + \left( \frac{1}{2} \right)^{N+2} \zeta_2
   *                  \frac{\Gamma(N+2) \Gamma(z)}{\Gamma(z+N+2)}
   *             + {\cal O}\left( \frac{\ln z}{z^{N+3}} \right)
   * \f]
   * Otherwise one uses the recurrence relation
   * \f[
   *    M_1(z) = - M_1(z+1) + M_2(z)
   * \f]
   * first.
   *
   */
  complex_d M1_helper(const complex_d & z)
  {
    if ( real(z) < 8.0 ) return M2_helper(z) - M1_helper(z+1.0);

    int Nmax = 10;

    complex_d res = pow(0.5, Nmax+2)*zeta2*Gamma(z)*Gamma(Nmax+2)/Gamma(z+2.0+ (double) Nmax);

    for ( int j=0; j<=Nmax; j++)
      {
	for ( int l=0; l<=j; l++ ) res += pow(0.5,j+1)*Gamma(j+1)/Gamma(l+1)/Gamma(j-l+1)*pow(-1,l)*M2_helper(z + (double) l);
      }

    return res;
  }

  /**
   *
   * The function is defined by
   * \f[
   *    M_2(z) = \int\limits_0^1 dx x^{z-2} \mbox{Li}_2(x)
   *           = \frac{\zeta_2}{z-1} - \frac{\psi(z)-\psi(1)}{(z-1)^2}
   * \f]
   *
   */
  complex_d M2_helper(const complex_d & z)
  {
    return zeta2/(z-1.0) - ( Psi(z)-Psi(1) )/pow(z-1.0,2);
  }

} // namespace pdf

